package com.mysql.jdbc;

import com.mysql.jdbc.CallableStatement;
import com.mysql.jdbc.PreparedStatement;
import com.mysql.jdbc.log.Log;
import com.mysql.jdbc.log.LogFactory;
import com.mysql.jdbc.log.NullLogger;
import com.mysql.jdbc.profiler.ProfileEventSink;
import com.mysql.jdbc.profiler.ProfilerEvent;
import com.mysql.jdbc.util.LRUCache;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.TimeZone;
import java.util.Timer;

/* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/ConnectionImpl.class */
public class ConnectionImpl extends ConnectionPropertiesImpl implements Connection {
    public static Map charsetMap;
    protected static final String DEFAULT_LOGGER_CLASS = "com.mysql.jdbc.log.StandardLogger";
    private static final int HISTOGRAM_BUCKETS = 20;
    private static Map mapTransIsolationNameToValue;
    private static Map roundRobinStatsMap;
    private static Timer cancelTimer;
    private static final Constructor JDBC_4_CONNECTION_CTOR;
    private boolean autoCommit;
    private Map cachedPreparedStatementParams;
    private String characterSetMetadata;
    private String characterSetResultsOnServer;
    private Map charsetConverterMap;
    private Map charsetToNumBytesMap;
    private long connectionCreationTimeMillis;
    private long connectionId;
    private String database;
    private java.sql.DatabaseMetaData dbmd;
    private TimeZone defaultTimeZone;
    private ProfileEventSink eventSink;
    private boolean executingFailoverReconnect;
    private boolean failedOver;
    private Throwable forceClosedReason;
    private Throwable forcedClosedLocation;
    private boolean hasIsolationLevels;
    private boolean hasQuotedIdentifiers;
    private String host;
    private List hostList;
    private int hostListSize;
    private String[] indexToCharsetMapping;
    private MysqlIO io;
    private boolean isClientTzUTC;
    private boolean isClosed;
    private boolean isInGlobalTx;
    private boolean isRunningOnJDK13;
    private int isolationLevel;
    private boolean isServerTzUTC;
    private long lastQueryFinishedTime;
    private Log log;
    private long longestQueryTimeMs;
    private boolean lowerCaseTableNames;
    private long masterFailTimeMillis;
    private int maxAllowedPacket;
    private long maximumNumberTablesAccessed;
    private boolean maxRowsChanged;
    private long metricsLastReportedMs;
    private long minimumNumberTablesAccessed;
    private final Object mutex;
    private String myURL;
    private boolean needsPing;
    private int netBufferLength;
    private boolean noBackslashEscapes;
    private long numberOfPreparedExecutes;
    private long numberOfPrepares;
    private long numberOfQueriesIssued;
    private long numberOfResultSetsCreated;
    private long[] numTablesMetricsHistBreakpoints;
    private int[] numTablesMetricsHistCounts;
    private long[] oldHistBreakpoints;
    private int[] oldHistCounts;
    private Map openStatements;
    private LRUCache parsedCallableStatementCache;
    private boolean parserKnowsUnicode;
    private String password;
    private long[] perfMetricsHistBreakpoints;
    private int[] perfMetricsHistCounts;
    private Throwable pointOfOrigin;
    private int port;
    private boolean preferSlaveDuringFailover;
    protected Properties props;
    private long queriesIssuedFailedOver;
    private boolean readInfoMsg;
    private boolean readOnly;
    protected LRUCache resultSetMetadataCache;
    private TimeZone serverTimezoneTZ;
    private Map serverVariables;
    private long shortestQueryTimeMs;
    private Map statementsUsingMaxRows;
    private double totalQueryTimeMs;
    private boolean transactionsSupported;
    private Map typeMap;
    private boolean useAnsiQuotes;
    private String user;
    private boolean useServerPreparedStmts;
    private LRUCache serverSideStatementCheckCache;
    private LRUCache serverSideStatementCache;
    private Calendar sessionCalendar;
    private Calendar utcCalendar;
    private String origHostToConnectTo;
    private int origPortToConnectTo;
    private String origDatabaseToConnectTo;
    private String errorMessageEncoding;
    private boolean usePlatformCharsetConverters;
    private boolean hasTriedMasterFlag;
    private String statementComment;
    static Class class$java$lang$String;
    static Class class$java$util$Timer;
    static Class class$java$util$Properties;
    static Class class$java$lang$Throwable;
    static Class class$java$sql$Blob;
    private static final Object CHARSET_CONVERTER_NOT_AVAILABLE_MARKER = new Object();
    private static final String LOGGER_INSTANCE_NAME = "MySQL";
    private static final Log NULL_LOGGER = new NullLogger(LOGGER_INSTANCE_NAME);
    private static final Map serverCollationByUrl = new HashMap();
    private static final Map serverConfigByUrl = new HashMap();

    /* JADX INFO: Access modifiers changed from: package-private */
    /* loaded from: com_mysql_jdbc.jar:com/mysql/jdbc/ConnectionImpl$CompoundCacheKey.class */
    public class CompoundCacheKey {
        String componentOne;
        String componentTwo;
        int hashCode;
        private final ConnectionImpl this$0;

        CompoundCacheKey(ConnectionImpl this$0, String partOne, String partTwo) {
            this.this$0 = this$0;
            this.componentOne = partOne;
            this.componentTwo = partTwo;
            this.hashCode = new StringBuffer().append(this.componentOne != null ? this.componentOne : "").append(this.componentTwo).toString().hashCode();
        }

        public boolean equals(Object obj) {
            boolean firstPartEqual;
            if (obj instanceof CompoundCacheKey) {
                CompoundCacheKey another = (CompoundCacheKey) obj;
                if (this.componentOne == null) {
                    firstPartEqual = another.componentOne == null;
                } else {
                    firstPartEqual = this.componentOne.equals(another.componentOne);
                }
                return firstPartEqual && this.componentTwo.equals(another.componentTwo);
            }
            return false;
        }

        public int hashCode() {
            return this.hashCode;
        }
    }

    static {
        boolean createdNamedTimer;
        Class<?> cls;
        Class<?> cls2;
        Class<?> cls3;
        Class<?> cls4;
        Class cls5;
        Class<?> cls6;
        mapTransIsolationNameToValue = null;
        mapTransIsolationNameToValue = new HashMap(8);
        mapTransIsolationNameToValue.put("READ-UNCOMMITED", Constants.integerValueOf(1));
        mapTransIsolationNameToValue.put("READ-UNCOMMITTED", Constants.integerValueOf(1));
        mapTransIsolationNameToValue.put("READ-COMMITTED", Constants.integerValueOf(2));
        mapTransIsolationNameToValue.put("REPEATABLE-READ", Constants.integerValueOf(4));
        mapTransIsolationNameToValue.put("SERIALIZABLE", Constants.integerValueOf(8));
        try {
            if (class$java$util$Timer == null) {
                cls5 = class$("java.util.Timer");
                class$java$util$Timer = cls5;
            } else {
                cls5 = class$java$util$Timer;
            }
            Class<?>[] clsArr = new Class[2];
            if (class$java$lang$String == null) {
                cls6 = class$("java.lang.String");
                class$java$lang$String = cls6;
            } else {
                cls6 = class$java$lang$String;
            }
            clsArr[0] = cls6;
            clsArr[1] = Boolean.TYPE;
            Constructor ctr = cls5.getConstructor(clsArr);
            cancelTimer = (Timer) ctr.newInstance("MySQL Statement Cancellation Timer", Boolean.TRUE);
            createdNamedTimer = true;
        } catch (Throwable th) {
            createdNamedTimer = false;
        }
        if (!createdNamedTimer) {
            cancelTimer = new Timer(true);
        }
        if (Util.isJdbc4()) {
            try {
                Class<?> cls7 = Class.forName("com.mysql.jdbc.JDBC4Connection");
                Class<?>[] clsArr2 = new Class[5];
                if (class$java$lang$String == null) {
                    cls = class$("java.lang.String");
                    class$java$lang$String = cls;
                } else {
                    cls = class$java$lang$String;
                }
                clsArr2[0] = cls;
                clsArr2[1] = Integer.TYPE;
                if (class$java$util$Properties == null) {
                    cls2 = class$("java.util.Properties");
                    class$java$util$Properties = cls2;
                } else {
                    cls2 = class$java$util$Properties;
                }
                clsArr2[2] = cls2;
                if (class$java$lang$String == null) {
                    cls3 = class$("java.lang.String");
                    class$java$lang$String = cls3;
                } else {
                    cls3 = class$java$lang$String;
                }
                clsArr2[3] = cls3;
                if (class$java$lang$String == null) {
                    cls4 = class$("java.lang.String");
                    class$java$lang$String = cls4;
                } else {
                    cls4 = class$java$lang$String;
                }
                clsArr2[4] = cls4;
                JDBC_4_CONNECTION_CTOR = cls7.getConstructor(clsArr2);
                return;
            } catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            } catch (NoSuchMethodException e2) {
                throw new RuntimeException(e2);
            } catch (SecurityException e3) {
                throw new RuntimeException(e3);
            }
        }
        JDBC_4_CONNECTION_CTOR = null;
    }

    static Class class$(String x0) {
        try {
            return Class.forName(x0);
        } catch (ClassNotFoundException x1) {
            throw new NoClassDefFoundError(x1.getMessage());
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public static SQLException appendMessageToException(SQLException sqlEx, String messageToAppend) {
        Class cls;
        Class cls2;
        String origMessage = sqlEx.getMessage();
        String sqlState = sqlEx.getSQLState();
        int vendorErrorCode = sqlEx.getErrorCode();
        StringBuffer messageBuf = new StringBuffer(origMessage.length() + messageToAppend.length());
        messageBuf.append(origMessage);
        messageBuf.append(messageToAppend);
        SQLException sqlExceptionWithNewMessage = SQLError.createSQLException(messageBuf.toString(), sqlState, vendorErrorCode);
        try {
            Class stackTraceElementClass = Class.forName("java.lang.StackTraceElement");
            Class stackTraceElementArrayClass = Array.newInstance((Class<?>) stackTraceElementClass, 0).getClass();
            if (class$java$lang$Throwable == null) {
                cls = class$("java.lang.Throwable");
                class$java$lang$Throwable = cls;
            } else {
                cls = class$java$lang$Throwable;
            }
            Method getStackTraceMethod = cls.getMethod("getStackTrace", new Class[0]);
            if (class$java$lang$Throwable == null) {
                cls2 = class$("java.lang.Throwable");
                class$java$lang$Throwable = cls2;
            } else {
                cls2 = class$java$lang$Throwable;
            }
            Method setStackTraceMethod = cls2.getMethod("setStackTrace", stackTraceElementArrayClass);
            if (getStackTraceMethod != null && setStackTraceMethod != null) {
                Object theStackTraceAsObject = getStackTraceMethod.invoke(sqlEx, new Object[0]);
                setStackTraceMethod.invoke(sqlExceptionWithNewMessage, theStackTraceAsObject);
            }
        } catch (NoClassDefFoundError e) {
        } catch (NoSuchMethodException e2) {
        } catch (Throwable th) {
        }
        return sqlExceptionWithNewMessage;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public static Timer getCancelTimer() {
        return cancelTimer;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public static Connection getInstance(String hostToConnectTo, int portToConnectTo, Properties info, String databaseToConnectTo, String url) throws SQLException {
        return !Util.isJdbc4() ? new ConnectionImpl(hostToConnectTo, portToConnectTo, info, databaseToConnectTo, url) : (Connection) Util.handleNewInstance(JDBC_4_CONNECTION_CTOR, new Object[]{hostToConnectTo, Constants.integerValueOf(portToConnectTo), info, databaseToConnectTo, url});
    }

    private static synchronized int getNextRoundRobinHostIndex(String url, List hostList) {
        if (roundRobinStatsMap == null) {
            roundRobinStatsMap = new HashMap();
        }
        int[] index = (int[]) roundRobinStatsMap.get(url);
        if (index == null) {
            index = new int[]{-1};
            roundRobinStatsMap.put(url, index);
        }
        int[] iArr = index;
        iArr[0] = iArr[0] + 1;
        if (index[0] >= hostList.size()) {
            index[0] = 0;
        }
        return index[0];
    }

    private static boolean nullSafeCompare(String s1, String s2) {
        if (s1 == null && s2 == null) {
            return true;
        }
        if (s1 == null && s2 != null) {
            return false;
        }
        return s1.equals(s2);
    }

    protected ConnectionImpl() {
        this.autoCommit = true;
        this.characterSetMetadata = null;
        this.characterSetResultsOnServer = null;
        this.charsetConverterMap = new HashMap(CharsetMapping.getNumberOfCharsetsConfigured());
        this.connectionCreationTimeMillis = 0L;
        this.database = null;
        this.dbmd = null;
        this.executingFailoverReconnect = false;
        this.failedOver = false;
        this.hasIsolationLevels = false;
        this.hasQuotedIdentifiers = false;
        this.host = null;
        this.hostList = null;
        this.hostListSize = 0;
        this.indexToCharsetMapping = CharsetMapping.INDEX_TO_CHARSET;
        this.io = null;
        this.isClientTzUTC = false;
        this.isClosed = true;
        this.isInGlobalTx = false;
        this.isRunningOnJDK13 = false;
        this.isolationLevel = 2;
        this.isServerTzUTC = false;
        this.lastQueryFinishedTime = 0L;
        this.log = NULL_LOGGER;
        this.longestQueryTimeMs = 0L;
        this.lowerCaseTableNames = false;
        this.masterFailTimeMillis = 0L;
        this.maxAllowedPacket = 65536;
        this.maximumNumberTablesAccessed = 0L;
        this.maxRowsChanged = false;
        this.minimumNumberTablesAccessed = Long.MAX_VALUE;
        this.mutex = new Object();
        this.myURL = null;
        this.needsPing = false;
        this.netBufferLength = 16384;
        this.noBackslashEscapes = false;
        this.numberOfPreparedExecutes = 0L;
        this.numberOfPrepares = 0L;
        this.numberOfQueriesIssued = 0L;
        this.numberOfResultSetsCreated = 0L;
        this.oldHistBreakpoints = null;
        this.oldHistCounts = null;
        this.parserKnowsUnicode = false;
        this.password = null;
        this.port = 3306;
        this.preferSlaveDuringFailover = false;
        this.props = null;
        this.queriesIssuedFailedOver = 0L;
        this.readInfoMsg = false;
        this.readOnly = false;
        this.serverTimezoneTZ = null;
        this.serverVariables = null;
        this.shortestQueryTimeMs = Long.MAX_VALUE;
        this.totalQueryTimeMs = 0.0d;
        this.transactionsSupported = false;
        this.useAnsiQuotes = false;
        this.user = null;
        this.useServerPreparedStmts = false;
        this.errorMessageEncoding = "Cp1252";
        this.hasTriedMasterFlag = false;
        this.statementComment = null;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public ConnectionImpl(String hostToConnectTo, int portToConnectTo, Properties info, String databaseToConnectTo, String url) throws SQLException {
        Class cls;
        this.autoCommit = true;
        this.characterSetMetadata = null;
        this.characterSetResultsOnServer = null;
        this.charsetConverterMap = new HashMap(CharsetMapping.getNumberOfCharsetsConfigured());
        this.connectionCreationTimeMillis = 0L;
        this.database = null;
        this.dbmd = null;
        this.executingFailoverReconnect = false;
        this.failedOver = false;
        this.hasIsolationLevels = false;
        this.hasQuotedIdentifiers = false;
        this.host = null;
        this.hostList = null;
        this.hostListSize = 0;
        this.indexToCharsetMapping = CharsetMapping.INDEX_TO_CHARSET;
        this.io = null;
        this.isClientTzUTC = false;
        this.isClosed = true;
        this.isInGlobalTx = false;
        this.isRunningOnJDK13 = false;
        this.isolationLevel = 2;
        this.isServerTzUTC = false;
        this.lastQueryFinishedTime = 0L;
        this.log = NULL_LOGGER;
        this.longestQueryTimeMs = 0L;
        this.lowerCaseTableNames = false;
        this.masterFailTimeMillis = 0L;
        this.maxAllowedPacket = 65536;
        this.maximumNumberTablesAccessed = 0L;
        this.maxRowsChanged = false;
        this.minimumNumberTablesAccessed = Long.MAX_VALUE;
        this.mutex = new Object();
        this.myURL = null;
        this.needsPing = false;
        this.netBufferLength = 16384;
        this.noBackslashEscapes = false;
        this.numberOfPreparedExecutes = 0L;
        this.numberOfPrepares = 0L;
        this.numberOfQueriesIssued = 0L;
        this.numberOfResultSetsCreated = 0L;
        this.oldHistBreakpoints = null;
        this.oldHistCounts = null;
        this.parserKnowsUnicode = false;
        this.password = null;
        this.port = 3306;
        this.preferSlaveDuringFailover = false;
        this.props = null;
        this.queriesIssuedFailedOver = 0L;
        this.readInfoMsg = false;
        this.readOnly = false;
        this.serverTimezoneTZ = null;
        this.serverVariables = null;
        this.shortestQueryTimeMs = Long.MAX_VALUE;
        this.totalQueryTimeMs = 0.0d;
        this.transactionsSupported = false;
        this.useAnsiQuotes = false;
        this.user = null;
        this.useServerPreparedStmts = false;
        this.errorMessageEncoding = "Cp1252";
        this.hasTriedMasterFlag = false;
        this.statementComment = null;
        this.charsetToNumBytesMap = new HashMap();
        this.connectionCreationTimeMillis = System.currentTimeMillis();
        this.pointOfOrigin = new Throwable();
        this.origHostToConnectTo = hostToConnectTo;
        this.origPortToConnectTo = portToConnectTo;
        this.origDatabaseToConnectTo = databaseToConnectTo;
        try {
            if (class$java$sql$Blob == null) {
                cls = class$("java.sql.Blob");
                class$java$sql$Blob = cls;
            } else {
                cls = class$java$sql$Blob;
            }
            cls.getMethod("truncate", Long.TYPE);
            this.isRunningOnJDK13 = false;
        } catch (NoSuchMethodException e) {
            this.isRunningOnJDK13 = true;
        }
        this.sessionCalendar = new GregorianCalendar();
        this.utcCalendar = new GregorianCalendar();
        this.utcCalendar.setTimeZone(TimeZone.getTimeZone("GMT"));
        this.log = LogFactory.getLogger(getLogger(), LOGGER_INSTANCE_NAME);
        this.defaultTimeZone = Util.getDefaultTimeZone();
        if ("GMT".equalsIgnoreCase(this.defaultTimeZone.getID())) {
            this.isClientTzUTC = true;
        } else {
            this.isClientTzUTC = false;
        }
        this.openStatements = new HashMap();
        this.serverVariables = new HashMap();
        this.hostList = new ArrayList();
        if (hostToConnectTo == null) {
            this.host = "localhost";
            this.hostList.add(this.host);
        } else if (hostToConnectTo.indexOf(44) != -1) {
            StringTokenizer hostTokenizer = new StringTokenizer(hostToConnectTo, ",", false);
            while (hostTokenizer.hasMoreTokens()) {
                this.hostList.add(hostTokenizer.nextToken().trim());
            }
        } else {
            this.host = hostToConnectTo;
            this.hostList.add(this.host);
        }
        this.hostListSize = this.hostList.size();
        this.port = portToConnectTo;
        this.database = databaseToConnectTo == null ? "" : databaseToConnectTo;
        this.myURL = url;
        this.user = info.getProperty(NonRegisteringDriver.USER_PROPERTY_KEY);
        this.password = info.getProperty(NonRegisteringDriver.PASSWORD_PROPERTY_KEY);
        if (this.user == null || this.user.equals("")) {
            this.user = "";
        }
        if (this.password == null) {
            this.password = "";
        }
        this.props = info;
        initializeDriverProperties(info);
        try {
            createNewIO(false);
            this.dbmd = getMetaData();
        } catch (SQLException ex) {
            cleanup(ex);
            throw ex;
        } catch (Exception ex2) {
            cleanup(ex2);
            StringBuffer mesg = new StringBuffer(128);
            if (getParanoid()) {
                mesg.append("Cannot connect to MySQL server on ");
                mesg.append(this.host);
                mesg.append(":");
                mesg.append(this.port);
                mesg.append(".\n\n");
                mesg.append("Make sure that there is a MySQL server ");
                mesg.append("running on the machine/port you are trying ");
                mesg.append("to connect to and that the machine this software is running on ");
                mesg.append("is able to connect to this host/port (i.e. not firewalled). ");
                mesg.append("Also make sure that the server has not been started with the --skip-networking ");
                mesg.append("flag.\n\n");
            } else {
                mesg.append("Unable to connect to database.");
            }
            mesg.append("Underlying exception: \n\n");
            mesg.append(ex2.getClass().getName());
            if (!getParanoid()) {
                mesg.append(Util.stackTraceToString(ex2));
            }
            throw SQLError.createSQLException(mesg.toString(), SQLError.SQL_STATE_COMMUNICATION_LINK_FAILURE);
        }
    }

    private void addToHistogram(int[] histogramCounts, long[] histogramBreakpoints, long value, int numberOfTimes, long currentLowerBound, long currentUpperBound) {
        if (histogramCounts == null) {
            createInitialHistogram(histogramBreakpoints, currentLowerBound, currentUpperBound);
        }
        for (int i = 0; i < HISTOGRAM_BUCKETS; i++) {
            if (histogramBreakpoints[i] >= value) {
                int i2 = i;
                histogramCounts[i2] = histogramCounts[i2] + numberOfTimes;
                return;
            }
        }
    }

    private void addToPerformanceHistogram(long value, int numberOfTimes) {
        checkAndCreatePerformanceHistogram();
        addToHistogram(this.perfMetricsHistCounts, this.perfMetricsHistBreakpoints, value, numberOfTimes, this.shortestQueryTimeMs == Long.MAX_VALUE ? 0L : this.shortestQueryTimeMs, this.longestQueryTimeMs);
    }

    private void addToTablesAccessedHistogram(long value, int numberOfTimes) {
        checkAndCreateTablesAccessedHistogram();
        addToHistogram(this.numTablesMetricsHistCounts, this.numTablesMetricsHistBreakpoints, value, numberOfTimes, this.minimumNumberTablesAccessed == Long.MAX_VALUE ? 0L : this.minimumNumberTablesAccessed, this.maximumNumberTablesAccessed);
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:55:0x0173
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    private void buildCollationMapping() throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 386
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.buildCollationMapping():void");
    }

    private boolean canHandleAsServerPreparedStatement(String sql) throws SQLException {
        if (sql == null || sql.length() == 0) {
            return true;
        }
        if (getCachePreparedStatements()) {
            synchronized (this.serverSideStatementCheckCache) {
                Boolean flag = (Boolean) this.serverSideStatementCheckCache.get(sql);
                if (flag != null) {
                    return flag.booleanValue();
                }
                boolean canHandle = canHandleAsServerPreparedStatementNoCache(sql);
                if (sql.length() < getPreparedStatementCacheSqlLimit()) {
                    this.serverSideStatementCheckCache.put(sql, canHandle ? Boolean.TRUE : Boolean.FALSE);
                }
                return canHandle;
            }
        }
        return canHandleAsServerPreparedStatementNoCache(sql);
    }

    private boolean canHandleAsServerPreparedStatementNoCache(String sql) throws SQLException {
        int limitStart;
        if (StringUtils.startsWithIgnoreCaseAndNonAlphaNumeric(sql, "CALL")) {
            return false;
        }
        boolean canHandleAsStatement = true;
        if (!versionMeetsMinimum(5, 0, 7) && (StringUtils.startsWithIgnoreCaseAndNonAlphaNumeric(sql, "SELECT") || StringUtils.startsWithIgnoreCaseAndNonAlphaNumeric(sql, "DELETE") || StringUtils.startsWithIgnoreCaseAndNonAlphaNumeric(sql, "INSERT") || StringUtils.startsWithIgnoreCaseAndNonAlphaNumeric(sql, "UPDATE") || StringUtils.startsWithIgnoreCaseAndNonAlphaNumeric(sql, "REPLACE"))) {
            int currentPos = 0;
            int statementLength = sql.length();
            int lastPosToLook = statementLength - 7;
            boolean allowBackslashEscapes = !this.noBackslashEscapes;
            char quoteChar = this.useAnsiQuotes ? '\"' : '\'';
            boolean foundLimitWithPlaceholder = false;
            while (currentPos < lastPosToLook && (limitStart = StringUtils.indexOfIgnoreCaseRespectQuotes(currentPos, sql, "LIMIT ", quoteChar, allowBackslashEscapes)) != -1) {
                currentPos = limitStart + 7;
                while (true) {
                    if (currentPos < statementLength) {
                        char c = sql.charAt(currentPos);
                        if (Character.isDigit(c) || Character.isWhitespace(c) || c == ',' || c == '?') {
                            if (c == '?') {
                                foundLimitWithPlaceholder = true;
                                break;
                            }
                            currentPos++;
                        }
                    }
                }
            }
            canHandleAsStatement = !foundLimitWithPlaceholder;
        } else if (StringUtils.startsWithIgnoreCaseAndWs(sql, "CREATE TABLE")) {
            canHandleAsStatement = false;
        } else if (StringUtils.startsWithIgnoreCaseAndWs(sql, "DO")) {
            canHandleAsStatement = false;
        } else if (StringUtils.startsWithIgnoreCaseAndWs(sql, "SET")) {
            canHandleAsStatement = false;
        }
        return canHandleAsStatement;
    }

    @Override // com.mysql.jdbc.Connection
    public void changeUser(String userName, String newPassword) throws SQLException {
        if (userName == null || userName.equals("")) {
            userName = "";
        }
        if (newPassword == null) {
            newPassword = "";
        }
        this.io.changeUser(userName, newPassword, this.database);
        this.user = userName;
        this.password = newPassword;
        if (versionMeetsMinimum(4, 1, 0)) {
            configureClientCharacterSet();
        }
        setupServerForTruncationChecks();
    }

    private boolean characterSetNamesMatches(String mysqlEncodingName) {
        return mysqlEncodingName != null && mysqlEncodingName.equalsIgnoreCase((String) this.serverVariables.get("character_set_client")) && mysqlEncodingName.equalsIgnoreCase((String) this.serverVariables.get("character_set_connection"));
    }

    private void checkAndCreatePerformanceHistogram() {
        if (this.perfMetricsHistCounts == null) {
            this.perfMetricsHistCounts = new int[HISTOGRAM_BUCKETS];
        }
        if (this.perfMetricsHistBreakpoints == null) {
            this.perfMetricsHistBreakpoints = new long[HISTOGRAM_BUCKETS];
        }
    }

    private void checkAndCreateTablesAccessedHistogram() {
        if (this.numTablesMetricsHistCounts == null) {
            this.numTablesMetricsHistCounts = new int[HISTOGRAM_BUCKETS];
        }
        if (this.numTablesMetricsHistBreakpoints == null) {
            this.numTablesMetricsHistBreakpoints = new long[HISTOGRAM_BUCKETS];
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void checkClosed() throws SQLException {
        if (this.isClosed) {
            StringBuffer messageBuf = new StringBuffer("No operations allowed after connection closed.");
            if (this.forcedClosedLocation != null || this.forceClosedReason != null) {
                messageBuf.append("Connection was implicitly closed ");
            }
            if (this.forcedClosedLocation != null) {
                messageBuf.append("\n\n at (stack trace):\n");
                messageBuf.append(Util.stackTraceToString(this.forcedClosedLocation));
            }
            if (this.forceClosedReason != null) {
                if (this.forcedClosedLocation != null) {
                    messageBuf.append("\n\nDue ");
                } else {
                    messageBuf.append("due ");
                }
                messageBuf.append("to underlying exception/error:\n");
                messageBuf.append(Util.stackTraceToString(this.forceClosedReason));
            }
            throw SQLError.createSQLException(messageBuf.toString(), SQLError.SQL_STATE_CONNECTION_NOT_OPEN);
        }
    }

    private void checkServerEncoding() throws SQLException {
        if (getUseUnicode() && getEncoding() != null) {
            return;
        }
        String serverEncoding = (String) this.serverVariables.get("character_set");
        if (serverEncoding == null) {
            serverEncoding = (String) this.serverVariables.get("character_set_server");
        }
        String mappedServerEncoding = null;
        if (serverEncoding != null) {
            mappedServerEncoding = CharsetMapping.getJavaEncodingForMysqlEncoding(serverEncoding.toUpperCase(Locale.ENGLISH), this);
        }
        if (!getUseUnicode() && mappedServerEncoding != null) {
            SingleByteCharsetConverter converter = getCharsetConverter(mappedServerEncoding);
            if (converter != null) {
                setUseUnicode(true);
                setEncoding(mappedServerEncoding);
                return;
            }
        }
        if (serverEncoding != null) {
            if (mappedServerEncoding == null && Character.isLowerCase(serverEncoding.charAt(0))) {
                char[] ach = serverEncoding.toCharArray();
                ach[0] = Character.toUpperCase(serverEncoding.charAt(0));
                setEncoding(new String(ach));
            }
            if (mappedServerEncoding == null) {
                throw SQLError.createSQLException(new StringBuffer().append("Unknown character encoding on server '").append(serverEncoding).append("', use 'characterEncoding=' property ").append(" to provide correct mapping").toString(), SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
            }
            try {
                "abc".getBytes(mappedServerEncoding);
                setEncoding(mappedServerEncoding);
                setUseUnicode(true);
            } catch (UnsupportedEncodingException e) {
                throw SQLError.createSQLException(new StringBuffer().append("The driver can not map the character encoding '").append(getEncoding()).append("' that your server is using ").append("to a character encoding your JVM understands. You ").append("can specify this mapping manually by adding \"useUnicode=true\" ").append("as well as \"characterEncoding=[an_encoding_your_jvm_understands]\" ").append("to your JDBC URL.").toString(), "0S100");
            }
        }
    }

    private void checkTransactionIsolationLevel() throws SQLException {
        String txIsolationName;
        Integer intTI;
        if (versionMeetsMinimum(4, 0, 3)) {
            txIsolationName = "tx_isolation";
        } else {
            txIsolationName = "transaction_isolation";
        }
        String s = (String) this.serverVariables.get(txIsolationName);
        if (s != null && (intTI = (Integer) mapTransIsolationNameToValue.get(s)) != null) {
            this.isolationLevel = intTI.intValue();
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void abortInternal() throws SQLException {
        this.io.forceClose();
        this.io = null;
        this.isClosed = true;
        cleanup(null);
    }

    private void cleanup(Throwable whyCleanedUp) {
        try {
            if (this.io != null && !isClosed()) {
                realClose(false, false, false, whyCleanedUp);
            } else if (this.io != null) {
                this.io.forceClose();
            }
        } catch (SQLException e) {
        }
        this.isClosed = true;
    }

    @Override // com.mysql.jdbc.Connection
    public void clearHasTriedMaster() {
        this.hasTriedMasterFlag = false;
    }

    @Override // java.sql.Connection
    public void clearWarnings() throws SQLException {
    }

    @Override // com.mysql.jdbc.Connection
    public PreparedStatement clientPrepareStatement(String sql) throws SQLException {
        return clientPrepareStatement(sql, MysqlErrorNumbers.ER_CANT_CREATE_TABLE, MysqlErrorNumbers.ER_DB_CREATE_EXISTS);
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement clientPrepareStatement(String sql, int autoGenKeyIndex) throws SQLException {
        java.sql.PreparedStatement pStmt = clientPrepareStatement(sql);
        ((PreparedStatement) pStmt).setRetrieveGeneratedKeys(autoGenKeyIndex == 1);
        return pStmt;
    }

    @Override // com.mysql.jdbc.Connection
    public PreparedStatement clientPrepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        return clientPrepareStatement(sql, resultSetType, resultSetConcurrency, true);
    }

    protected PreparedStatement clientPrepareStatement(String sql, int resultSetType, int resultSetConcurrency, boolean processEscapeCodesIfNeeded) throws SQLException {
        PreparedStatement pStmt;
        checkClosed();
        String nativeSql = (processEscapeCodesIfNeeded && getProcessEscapeCodesForPrepStmts()) ? nativeSQL(sql) : sql;
        if (getCachePreparedStatements()) {
            synchronized (this.cachedPreparedStatementParams) {
                PreparedStatement.ParseInfo pStmtInfo = (PreparedStatement.ParseInfo) this.cachedPreparedStatementParams.get(nativeSql);
                if (pStmtInfo == null) {
                    pStmt = PreparedStatement.getInstance(this, nativeSql, this.database);
                    PreparedStatement.ParseInfo parseInfo = pStmt.getParseInfo();
                    if (parseInfo.statementLength < getPreparedStatementCacheSqlLimit()) {
                        if (this.cachedPreparedStatementParams.size() >= getPreparedStatementCacheSize()) {
                            long lruTime = Long.MAX_VALUE;
                            String oldestSql = null;
                            for (String sqlKey : this.cachedPreparedStatementParams.keySet()) {
                                PreparedStatement.ParseInfo lruInfo = (PreparedStatement.ParseInfo) this.cachedPreparedStatementParams.get(sqlKey);
                                if (lruInfo.lastUsed < lruTime) {
                                    lruTime = lruInfo.lastUsed;
                                    oldestSql = sqlKey;
                                }
                            }
                            if (oldestSql != null) {
                                this.cachedPreparedStatementParams.remove(oldestSql);
                            }
                        }
                        this.cachedPreparedStatementParams.put(nativeSql, pStmt.getParseInfo());
                    }
                } else {
                    pStmtInfo.lastUsed = System.currentTimeMillis();
                    pStmt = new PreparedStatement(this, nativeSql, this.database, pStmtInfo);
                }
            }
        } else {
            pStmt = PreparedStatement.getInstance(this, nativeSql, this.database);
        }
        pStmt.setResultSetType(resultSetType);
        pStmt.setResultSetConcurrency(resultSetConcurrency);
        return pStmt;
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement clientPrepareStatement(String sql, int[] autoGenKeyIndexes) throws SQLException {
        PreparedStatement pStmt = clientPrepareStatement(sql);
        pStmt.setRetrieveGeneratedKeys(autoGenKeyIndexes != null && autoGenKeyIndexes.length > 0);
        return pStmt;
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement clientPrepareStatement(String sql, String[] autoGenKeyColNames) throws SQLException {
        PreparedStatement pStmt = clientPrepareStatement(sql);
        pStmt.setRetrieveGeneratedKeys(autoGenKeyColNames != null && autoGenKeyColNames.length > 0);
        return pStmt;
    }

    @Override // java.sql.Connection, java.lang.AutoCloseable
    public void close() throws SQLException {
        realClose(true, true, false, null);
    }

    private void closeAllOpenStatements() throws SQLException {
        SQLException postponedException = null;
        if (this.openStatements != null) {
            List currentlyOpenStatements = new ArrayList();
            Iterator iter = this.openStatements.keySet().iterator();
            while (iter.hasNext()) {
                currentlyOpenStatements.add(iter.next());
            }
            int numStmts = currentlyOpenStatements.size();
            for (int i = 0; i < numStmts; i++) {
                StatementImpl stmt = (StatementImpl) currentlyOpenStatements.get(i);
                try {
                    stmt.realClose(false, true);
                } catch (SQLException sqlEx) {
                    postponedException = sqlEx;
                }
            }
            if (postponedException != null) {
                throw postponedException;
            }
        }
    }

    private void closeStatement(java.sql.Statement stmt) {
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
            }
        }
    }

    @Override // java.sql.Connection
    public void commit() throws SQLException {
        synchronized (getMutex()) {
            try {
                checkClosed();
                try {
                    if (this.autoCommit && !getRelaxAutoCommit()) {
                        throw SQLError.createSQLException("Can't call commit when autocommit=true");
                    }
                    if (this.transactionsSupported) {
                        if (getUseLocalSessionState() && versionMeetsMinimum(5, 0, 0) && !this.io.inTransactionOnServer()) {
                            return;
                        } else {
                            execSQL(null, "commit", -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                        }
                    }
                    this.needsPing = getReconnectAtTxEnd();
                } catch (SQLException sqlException) {
                    if (SQLError.SQL_STATE_COMMUNICATION_LINK_FAILURE.equals(sqlException.getSQLState())) {
                        throw SQLError.createSQLException("Communications link failure during commit(). Transaction resolution unknown.", "08007");
                    }
                    throw sqlException;
                }
            } finally {
                this.needsPing = getReconnectAtTxEnd();
            }
        }
    }

    private void configureCharsetProperties() throws SQLException {
        if (getEncoding() != null) {
            try {
                "abc".getBytes(getEncoding());
            } catch (UnsupportedEncodingException e) {
                String oldEncoding = getEncoding();
                setEncoding(CharsetMapping.getJavaEncodingForMysqlEncoding(oldEncoding, this));
                if (getEncoding() == null) {
                    throw SQLError.createSQLException(new StringBuffer().append("Java does not support the MySQL character encoding  encoding '").append(oldEncoding).append("'.").toString(), SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
                }
                try {
                    "abc".getBytes(getEncoding());
                } catch (UnsupportedEncodingException e2) {
                    throw SQLError.createSQLException(new StringBuffer().append("Unsupported character encoding '").append(getEncoding()).append("'.").toString(), SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
                }
            }
        }
    }

    private boolean configureClientCharacterSet() throws SQLException {
        String mysqlEncodingName;
        String realJavaEncoding = getEncoding();
        boolean characterSetAlreadyConfigured = false;
        try {
            if (versionMeetsMinimum(4, 1, 0)) {
                characterSetAlreadyConfigured = true;
                setUseUnicode(true);
                configureCharsetProperties();
                realJavaEncoding = getEncoding();
                try {
                    if (this.props != null && this.props.getProperty("com.mysql.jdbc.faultInjection.serverCharsetIndex") != null) {
                        this.io.serverCharsetIndex = Integer.parseInt(this.props.getProperty("com.mysql.jdbc.faultInjection.serverCharsetIndex"));
                    }
                    String serverEncodingToSet = CharsetMapping.INDEX_TO_CHARSET[this.io.serverCharsetIndex];
                    if (serverEncodingToSet == null || serverEncodingToSet.length() == 0) {
                        if (realJavaEncoding != null) {
                            setEncoding(realJavaEncoding);
                        } else {
                            throw SQLError.createSQLException(new StringBuffer().append("Unknown initial character set index '").append(this.io.serverCharsetIndex).append("' received from server. Initial client character set can be forced via the 'characterEncoding' property.").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
                        }
                    }
                    if (versionMeetsMinimum(4, 1, 0) && "ISO8859_1".equalsIgnoreCase(serverEncodingToSet)) {
                        serverEncodingToSet = "Cp1252";
                    }
                    setEncoding(serverEncodingToSet);
                } catch (ArrayIndexOutOfBoundsException e) {
                    if (realJavaEncoding != null) {
                        setEncoding(realJavaEncoding);
                    } else {
                        throw SQLError.createSQLException(new StringBuffer().append("Unknown initial character set index '").append(this.io.serverCharsetIndex).append("' received from server. Initial client character set can be forced via the 'characterEncoding' property.").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
                    }
                }
                if (getEncoding() == null) {
                    setEncoding("ISO8859_1");
                }
                if (getUseUnicode()) {
                    if (realJavaEncoding != null) {
                        if (realJavaEncoding.equalsIgnoreCase("UTF-8") || realJavaEncoding.equalsIgnoreCase("UTF8")) {
                            if (!getUseOldUTF8Behavior() && !characterSetNamesMatches("utf8")) {
                                execSQL(null, "SET NAMES utf8", -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                            }
                            setEncoding(realJavaEncoding);
                        } else {
                            String mysqlEncodingName2 = CharsetMapping.getMysqlEncodingForJavaEncoding(realJavaEncoding.toUpperCase(Locale.ENGLISH), this);
                            if (mysqlEncodingName2 != null && !characterSetNamesMatches(mysqlEncodingName2)) {
                                execSQL(null, new StringBuffer().append("SET NAMES ").append(mysqlEncodingName2).toString(), -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                            }
                            setEncoding(realJavaEncoding);
                        }
                    } else if (getEncoding() != null) {
                        String mysqlEncodingName3 = CharsetMapping.getMysqlEncodingForJavaEncoding(getEncoding().toUpperCase(Locale.ENGLISH), this);
                        if (!characterSetNamesMatches(mysqlEncodingName3)) {
                            execSQL(null, new StringBuffer().append("SET NAMES ").append(mysqlEncodingName3).toString(), -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                        }
                        realJavaEncoding = getEncoding();
                    }
                }
                if (getCharacterSetResults() == null) {
                    if (this.serverVariables.get("character_set_results") != null) {
                        execSQL(null, "SET character_set_results = NULL", -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                    }
                } else {
                    String charsetResults = getCharacterSetResults();
                    if ("UTF-8".equalsIgnoreCase(charsetResults) || "UTF8".equalsIgnoreCase(charsetResults)) {
                        mysqlEncodingName = "utf8";
                    } else {
                        mysqlEncodingName = CharsetMapping.getMysqlEncodingForJavaEncoding(charsetResults.toUpperCase(Locale.ENGLISH), this);
                    }
                    if (!mysqlEncodingName.equalsIgnoreCase((String) this.serverVariables.get("character_set_results"))) {
                        StringBuffer setBuf = new StringBuffer("SET character_set_results = ".length() + mysqlEncodingName.length());
                        setBuf.append("SET character_set_results = ").append(mysqlEncodingName);
                        execSQL(null, setBuf.toString(), -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                    }
                }
                if (getConnectionCollation() != null) {
                    StringBuffer setBuf2 = new StringBuffer("SET collation_connection = ".length() + getConnectionCollation().length());
                    setBuf2.append("SET collation_connection = ").append(getConnectionCollation());
                    execSQL(null, setBuf2.toString(), -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                }
            } else {
                realJavaEncoding = getEncoding();
            }
            return characterSetAlreadyConfigured;
        } finally {
            setEncoding(realJavaEncoding);
        }
    }

    private void configureTimezone() throws SQLException {
        String configuredTimeZoneOnServer = (String) this.serverVariables.get("timezone");
        if (configuredTimeZoneOnServer == null) {
            configuredTimeZoneOnServer = (String) this.serverVariables.get("time_zone");
            if ("SYSTEM".equalsIgnoreCase(configuredTimeZoneOnServer)) {
                configuredTimeZoneOnServer = (String) this.serverVariables.get("system_time_zone");
            }
        }
        if (getUseTimezone() && configuredTimeZoneOnServer != null) {
            String canoncicalTimezone = getServerTimezone();
            if (canoncicalTimezone == null || canoncicalTimezone.length() == 0) {
                String serverTimezoneStr = configuredTimeZoneOnServer;
                try {
                    canoncicalTimezone = TimeUtil.getCanoncialTimezone(serverTimezoneStr);
                    if (canoncicalTimezone == null) {
                        throw SQLError.createSQLException(new StringBuffer().append("Can't map timezone '").append(serverTimezoneStr).append("' to ").append(" canonical timezone.").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
                    }
                } catch (IllegalArgumentException iae) {
                    throw SQLError.createSQLException(iae.getMessage(), SQLError.SQL_STATE_GENERAL_ERROR);
                }
            }
            this.serverTimezoneTZ = TimeZone.getTimeZone(canoncicalTimezone);
            if (!canoncicalTimezone.equalsIgnoreCase("GMT") && this.serverTimezoneTZ.getID().equals("GMT")) {
                throw SQLError.createSQLException(new StringBuffer().append("No timezone mapping entry for '").append(canoncicalTimezone).append("'").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
            }
            if ("GMT".equalsIgnoreCase(this.serverTimezoneTZ.getID())) {
                this.isServerTzUTC = true;
            } else {
                this.isServerTzUTC = false;
            }
        }
    }

    private void createInitialHistogram(long[] breakpoints, long lowerBound, long upperBound) {
        double bucketSize = ((upperBound - lowerBound) / 20.0d) * 1.25d;
        if (bucketSize < 1.0d) {
            bucketSize = 1.0d;
        }
        for (int i = 0; i < HISTOGRAM_BUCKETS; i++) {
            breakpoints[i] = lowerBound;
            lowerBound = (long) (lowerBound + bucketSize);
        }
    }

    /*  JADX ERROR: NullPointerException in pass: RegionMakerVisitor
        java.lang.NullPointerException: Cannot invoke "java.util.List.isEmpty()" because "s" is null
        	at jadx.core.utils.BlockUtils.getNextBlock(BlockUtils.java:411)
        	at jadx.core.dex.visitors.regions.RegionMaker.traverse(RegionMaker.java:172)
        	at jadx.core.dex.visitors.regions.RegionMaker.makeRegion(RegionMaker.java:91)
        	at jadx.core.dex.visitors.regions.RegionMaker.processIf(RegionMaker.java:735)
        	at jadx.core.dex.visitors.regions.RegionMaker.traverse(RegionMaker.java:152)
        	at jadx.core.dex.visitors.regions.RegionMaker.makeRegion(RegionMaker.java:91)
        	at jadx.core.dex.visitors.regions.RegionMaker.processExcHandler(RegionMaker.java:1110)
        	at jadx.core.dex.visitors.regions.RegionMaker.processTryCatchBlocks(RegionMaker.java:1046)
        	at jadx.core.dex.visitors.regions.RegionMakerVisitor.visit(RegionMakerVisitor.java:55)
        */
    protected void createNewIO(boolean r12) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 1279
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.createNewIO(boolean):void");
    }

    private void createPreparedStatementCaches() {
        int cacheSize = getPreparedStatementCacheSize();
        this.cachedPreparedStatementParams = new HashMap(cacheSize);
        this.serverSideStatementCheckCache = new LRUCache(cacheSize);
        this.serverSideStatementCache = new LRUCache(this, cacheSize) { // from class: com.mysql.jdbc.ConnectionImpl.1
            private final ConnectionImpl this$0;

            {
                this.this$0 = this;
            }

            protected boolean removeEldestEntry(Map.Entry eldest) {
                if (this.maxElements <= 1) {
                    return false;
                }
                boolean removeIt = super.removeEldestEntry(eldest);
                if (removeIt) {
                    ServerPreparedStatement ps = (ServerPreparedStatement) eldest.getValue();
                    ps.isCached = false;
                    ps.setClosed(false);
                    try {
                        ps.close();
                    } catch (SQLException e) {
                    }
                }
                return removeIt;
            }
        };
    }

    @Override // java.sql.Connection
    public java.sql.Statement createStatement() throws SQLException {
        return createStatement(MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS);
    }

    @Override // java.sql.Connection
    public java.sql.Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
        checkClosed();
        StatementImpl stmt = new StatementImpl(this, this.database);
        stmt.setResultSetType(resultSetType);
        stmt.setResultSetConcurrency(resultSetConcurrency);
        return stmt;
    }

    @Override // java.sql.Connection
    public java.sql.Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        if (getPedantic() && resultSetHoldability != 1) {
            throw SQLError.createSQLException("HOLD_CUSRORS_OVER_COMMIT is only supported holdability level", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        return createStatement(resultSetType, resultSetConcurrency);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void dumpTestcaseQuery(String query) {
        System.err.println(query);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public Connection duplicate() throws SQLException {
        return new ConnectionImpl(this.origHostToConnectTo, this.origPortToConnectTo, this.props, this.origDatabaseToConnectTo, this.myURL);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public ResultSetInternalMethods execSQL(StatementImpl callingStatement, String sql, int maxRows, Buffer packet, int resultSetType, int resultSetConcurrency, boolean streamResults, String catalog, Field[] cachedMetadata) throws SQLException {
        return execSQL(callingStatement, sql, maxRows, packet, resultSetType, resultSetConcurrency, streamResults, catalog, cachedMetadata, false);
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* JADX WARN: Code restructure failed: missing block: B:63:0x024a, code lost:
    
        if (getMaintainTimeStats() != false) goto L97;
     */
    /* JADX WARN: Code restructure failed: missing block: B:64:0x024d, code lost:
    
        r12.lastQueryFinishedTime = java.lang.System.currentTimeMillis();
     */
    /* JADX WARN: Code restructure failed: missing block: B:66:0x0258, code lost:
    
        if (r12.failedOver == false) goto L101;
     */
    /* JADX WARN: Code restructure failed: missing block: B:67:0x025b, code lost:
    
        r12.queriesIssuedFailedOver++;
     */
    /* JADX WARN: Code restructure failed: missing block: B:69:0x0269, code lost:
    
        if (getGatherPerformanceMetrics() == false) goto L104;
     */
    /* JADX WARN: Code restructure failed: missing block: B:70:0x026c, code lost:
    
        r0 = java.lang.System.currentTimeMillis() - r24;
        registerQueryExecutionTime(r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:74:0x012b, code lost:
    
        return r0;
     */
    /* JADX WARN: Code restructure failed: missing block: B:77:0x024a, code lost:
    
        if (getMaintainTimeStats() == false) goto L98;
     */
    /* JADX WARN: Code restructure failed: missing block: B:78:0x024d, code lost:
    
        r12.lastQueryFinishedTime = java.lang.System.currentTimeMillis();
     */
    /* JADX WARN: Code restructure failed: missing block: B:80:0x0258, code lost:
    
        if (r12.failedOver == false) goto L101;
     */
    /* JADX WARN: Code restructure failed: missing block: B:81:0x025b, code lost:
    
        r12.queriesIssuedFailedOver++;
     */
    /* JADX WARN: Code restructure failed: missing block: B:83:0x0269, code lost:
    
        if (getGatherPerformanceMetrics() == false) goto L104;
     */
    /* JADX WARN: Code restructure failed: missing block: B:84:0x026c, code lost:
    
        r0 = java.lang.System.currentTimeMillis() - r24;
        registerQueryExecutionTime(r0);
     */
    /* JADX WARN: Code restructure failed: missing block: B:86:0x0243, code lost:
    
        throw r30;
     */
    /* JADX WARN: Finally extract failed */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    public com.mysql.jdbc.ResultSetInternalMethods execSQL(com.mysql.jdbc.StatementImpl r13, java.lang.String r14, int r15, com.mysql.jdbc.Buffer r16, int r17, int r18, boolean r19, java.lang.String r20, com.mysql.jdbc.Field[] r21, boolean r22) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 644
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.execSQL(com.mysql.jdbc.StatementImpl, java.lang.String, int, com.mysql.jdbc.Buffer, int, int, boolean, java.lang.String, com.mysql.jdbc.Field[], boolean):com.mysql.jdbc.ResultSetInternalMethods");
    }

    protected String extractSqlFromPacket(String possibleSqlQuery, Buffer queryPacket, int endOfQueryPacketPosition) throws SQLException {
        String extractedSql = null;
        if (possibleSqlQuery != null) {
            if (possibleSqlQuery.length() > getMaxQuerySizeToLog()) {
                StringBuffer truncatedQueryBuf = new StringBuffer(possibleSqlQuery.substring(0, getMaxQuerySizeToLog()));
                truncatedQueryBuf.append(Messages.getString("MysqlIO.25"));
                extractedSql = truncatedQueryBuf.toString();
            } else {
                extractedSql = possibleSqlQuery;
            }
        }
        if (extractedSql == null) {
            int extractPosition = endOfQueryPacketPosition;
            boolean truncated = false;
            if (endOfQueryPacketPosition > getMaxQuerySizeToLog()) {
                extractPosition = getMaxQuerySizeToLog();
                truncated = true;
            }
            extractedSql = new String(queryPacket.getByteBuffer(), 5, extractPosition - 5);
            if (truncated) {
                extractedSql = new StringBuffer().append(extractedSql).append(Messages.getString("MysqlIO.25")).toString();
            }
        }
        return extractedSql;
    }

    protected void finalize() throws Throwable {
        cleanup(null);
        super.finalize();
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public StringBuffer generateConnectionCommentBlock(StringBuffer buf) {
        buf.append("/* conn id ");
        buf.append(getId());
        buf.append(" */ ");
        return buf;
    }

    @Override // com.mysql.jdbc.Connection
    public int getActiveStatementCount() {
        int size;
        if (this.openStatements != null) {
            synchronized (this.openStatements) {
                size = this.openStatements.size();
            }
            return size;
        }
        return 0;
    }

    @Override // java.sql.Connection
    public boolean getAutoCommit() throws SQLException {
        return this.autoCommit;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public Calendar getCalendarInstanceForSessionOrNew() {
        if (getDynamicCalendars()) {
            return Calendar.getInstance();
        }
        return getSessionLockedCalendar();
    }

    @Override // java.sql.Connection
    public String getCatalog() throws SQLException {
        return this.database;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public String getCharacterSetMetadata() {
        return this.characterSetMetadata;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public SingleByteCharsetConverter getCharsetConverter(String javaEncodingName) throws SQLException {
        if (javaEncodingName == null || this.usePlatformCharsetConverters) {
            return null;
        }
        synchronized (this.charsetConverterMap) {
            Object asObject = this.charsetConverterMap.get(javaEncodingName);
            if (asObject == CHARSET_CONVERTER_NOT_AVAILABLE_MARKER) {
                return null;
            }
            SingleByteCharsetConverter converter = (SingleByteCharsetConverter) asObject;
            if (converter == null) {
                try {
                    converter = SingleByteCharsetConverter.getInstance(javaEncodingName, this);
                    if (converter == null) {
                        this.charsetConverterMap.put(javaEncodingName, CHARSET_CONVERTER_NOT_AVAILABLE_MARKER);
                    } else {
                        this.charsetConverterMap.put(javaEncodingName, converter);
                    }
                } catch (UnsupportedEncodingException e) {
                    this.charsetConverterMap.put(javaEncodingName, CHARSET_CONVERTER_NOT_AVAILABLE_MARKER);
                    converter = null;
                }
            }
            return converter;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public String getCharsetNameForIndex(int charsetIndex) throws SQLException {
        String charsetName;
        if (getUseOldUTF8Behavior()) {
            return getEncoding();
        }
        if (charsetIndex != -1) {
            try {
                charsetName = this.indexToCharsetMapping[charsetIndex];
                if (("sjis".equalsIgnoreCase(charsetName) || "MS932".equalsIgnoreCase(charsetName)) && CharsetMapping.isAliasForSjis(getEncoding())) {
                    charsetName = getEncoding();
                }
                if (charsetName == null) {
                    charsetName = getEncoding();
                }
            } catch (ArrayIndexOutOfBoundsException e) {
                throw SQLError.createSQLException(new StringBuffer().append("Unknown character set index for field '").append(charsetIndex).append("' received from server.").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
            }
        } else {
            charsetName = getEncoding();
        }
        return charsetName;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public TimeZone getDefaultTimeZone() {
        return this.defaultTimeZone;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public String getErrorMessageEncoding() {
        return this.errorMessageEncoding;
    }

    @Override // java.sql.Connection
    public int getHoldability() throws SQLException {
        return 2;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public long getId() {
        return this.connectionId;
    }

    @Override // com.mysql.jdbc.Connection
    public long getIdleFor() {
        if (this.lastQueryFinishedTime == 0) {
            return 0L;
        }
        long now = System.currentTimeMillis();
        long idleTime = now - this.lastQueryFinishedTime;
        return idleTime;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public MysqlIO getIO() throws SQLException {
        if (this.io == null || this.isClosed) {
            throw SQLError.createSQLException("Operation not allowed on closed connection", SQLError.SQL_STATE_CONNECTION_NOT_OPEN);
        }
        return this.io;
    }

    @Override // com.mysql.jdbc.Connection
    public Log getLog() throws SQLException {
        return this.log;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getMaxAllowedPacket() {
        return this.maxAllowedPacket;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* JADX WARN: Code restructure failed: missing block: B:35:0x00a5, code lost:
    
        if (r11 == null) goto L25;
     */
    /* JADX WARN: Code restructure failed: missing block: B:36:0x00a8, code lost:
    
        r11.close();
     */
    /* JADX WARN: Code restructure failed: missing block: B:38:0x00b4, code lost:
    
        if (r10 == null) goto L28;
     */
    /* JADX WARN: Code restructure failed: missing block: B:39:0x00b7, code lost:
    
        r10.close();
     */
    /* JADX WARN: Code restructure failed: missing block: B:41:0x00a0, code lost:
    
        throw r12;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    public int getMaxBytesPerChar(java.lang.String r6) throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 236
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.getMaxBytesPerChar(java.lang.String):int");
    }

    @Override // java.sql.Connection
    public java.sql.DatabaseMetaData getMetaData() throws SQLException {
        checkClosed();
        return DatabaseMetaData.getInstance(this, this.database);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public java.sql.Statement getMetadataSafeStatement() throws SQLException {
        java.sql.Statement stmt = createStatement();
        if (stmt.getMaxRows() != 0) {
            stmt.setMaxRows(0);
        }
        stmt.setEscapeProcessing(false);
        return stmt;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public Object getMutex() throws SQLException {
        if (this.io == null) {
            throw SQLError.createSQLException("Connection.close() has already been called. Invalid operation in this state.", SQLError.SQL_STATE_CONNECTION_NOT_OPEN);
        }
        reportMetricsIfNeeded();
        return this.mutex;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getNetBufferLength() {
        return this.netBufferLength;
    }

    @Override // com.mysql.jdbc.Connection
    public String getServerCharacterEncoding() {
        return (String) this.serverVariables.get("character_set");
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getServerMajorVersion() {
        return this.io.getServerMajorVersion();
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public int getServerMinorVersion() {
        return this.io.getServerMinorVersion();
    }

    int getServerSubMinorVersion() {
        return this.io.getServerSubMinorVersion();
    }

    @Override // com.mysql.jdbc.Connection
    public TimeZone getServerTimezoneTZ() {
        return this.serverTimezoneTZ;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String getServerVariable(String variableName) {
        if (this.serverVariables != null) {
            return (String) this.serverVariables.get(variableName);
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String getServerVersion() {
        return this.io.getServerVersion();
    }

    protected Calendar getSessionLockedCalendar() {
        return this.sessionCalendar;
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:24:0x00cc
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    @Override // java.sql.Connection
    public int getTransactionIsolation() throws java.sql.SQLException {
        /*
            r5 = this;
            r0 = r5
            boolean r0 = r0.hasIsolationLevels
            if (r0 == 0) goto Ld2
            r0 = r5
            boolean r0 = r0.getUseLocalSessionState()
            if (r0 != 0) goto Ld2
            r0 = 0
            r6 = r0
            r0 = 0
            r7 = r0
            r0 = r5
            java.sql.Statement r0 = r0.getMetadataSafeStatement()     // Catch: java.lang.Throwable -> La4
            r6 = r0
            r0 = 0
            r8 = r0
            r0 = 0
            r9 = r0
            r0 = r5
            r1 = 4
            r2 = 0
            r3 = 3
            boolean r0 = r0.versionMeetsMinimum(r1, r2, r3)     // Catch: java.lang.Throwable -> La4
            if (r0 == 0) goto L30
            java.lang.String r0 = "SELECT @@session.tx_isolation"
            r8 = r0
            r0 = 1
            r9 = r0
            goto L37
        L30:
            java.lang.String r0 = "SHOW VARIABLES LIKE 'transaction_isolation'"
            r8 = r0
            r0 = 2
            r9 = r0
        L37:
            r0 = r6
            r1 = r8
            java.sql.ResultSet r0 = r0.executeQuery(r1)     // Catch: java.lang.Throwable -> La4
            r7 = r0
            r0 = r7
            boolean r0 = r0.next()     // Catch: java.lang.Throwable -> La4
            if (r0 == 0) goto L9a
            r0 = r7
            r1 = r9
            java.lang.String r0 = r0.getString(r1)     // Catch: java.lang.Throwable -> La4
            r10 = r0
            r0 = r10
            if (r0 == 0) goto L78
            java.util.Map r0 = com.mysql.jdbc.ConnectionImpl.mapTransIsolationNameToValue     // Catch: java.lang.Throwable -> La4
            r1 = r10
            java.lang.Object r0 = r0.get(r1)     // Catch: java.lang.Throwable -> La4
            java.lang.Integer r0 = (java.lang.Integer) r0     // Catch: java.lang.Throwable -> La4
            r11 = r0
            r0 = r11
            if (r0 == 0) goto L78
            r0 = r11
            int r0 = r0.intValue()     // Catch: java.lang.Throwable -> La4
            r12 = r0
            r0 = jsr -> Lac
        L75:
            r1 = r12
            return r1
        L78:
            java.lang.StringBuffer r0 = new java.lang.StringBuffer     // Catch: java.lang.Throwable -> La4
            r1 = r0
            r1.<init>()     // Catch: java.lang.Throwable -> La4
            java.lang.String r1 = "Could not map transaction isolation '"
            java.lang.StringBuffer r0 = r0.append(r1)     // Catch: java.lang.Throwable -> La4
            r1 = r10
            java.lang.StringBuffer r0 = r0.append(r1)     // Catch: java.lang.Throwable -> La4
            java.lang.String r1 = " to a valid JDBC level."
            java.lang.StringBuffer r0 = r0.append(r1)     // Catch: java.lang.Throwable -> La4
            java.lang.String r0 = r0.toString()     // Catch: java.lang.Throwable -> La4
            java.lang.String r1 = "S1000"
            java.sql.SQLException r0 = com.mysql.jdbc.SQLError.createSQLException(r0, r1)     // Catch: java.lang.Throwable -> La4
            throw r0     // Catch: java.lang.Throwable -> La4
        L9a:
            java.lang.String r0 = "Could not retrieve transaction isolation level from server"
            java.lang.String r1 = "S1000"
            java.sql.SQLException r0 = com.mysql.jdbc.SQLError.createSQLException(r0, r1)     // Catch: java.lang.Throwable -> La4
            throw r0     // Catch: java.lang.Throwable -> La4
        La4:
            r13 = move-exception
            r0 = jsr -> Lac
        La9:
            r1 = r13
            throw r1
        Lac:
            r14 = r0
            r0 = r7
            if (r0 == 0) goto Lbf
            r0 = r7
            r0.close()     // Catch: java.lang.Exception -> Lbb
            goto Lbd
        Lbb:
            r15 = move-exception
        Lbd:
            r0 = 0
            r7 = r0
        Lbf:
            r0 = r6
            if (r0 == 0) goto Ld0
            r0 = r6
            r0.close()     // Catch: java.lang.Exception -> Lcc
            goto Lce
        Lcc:
            r15 = move-exception
        Lce:
            r0 = 0
            r6 = r0
        Ld0:
            ret r14
        Ld2:
            r0 = r5
            int r0 = r0.isolationLevel
            return r0
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.getTransactionIsolation():int");
    }

    @Override // java.sql.Connection
    public synchronized Map getTypeMap() throws SQLException {
        if (this.typeMap == null) {
            this.typeMap = new HashMap();
        }
        return this.typeMap;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String getURL() {
        return this.myURL;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public String getUser() {
        return this.user;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public Calendar getUtcCalendar() {
        return this.utcCalendar;
    }

    @Override // java.sql.Connection
    public SQLWarning getWarnings() throws SQLException {
        return null;
    }

    public boolean hasSameProperties(Connection c) {
        return this.props.equals(((ConnectionImpl) c).props);
    }

    @Override // com.mysql.jdbc.Connection
    public boolean hasTriedMaster() {
        return this.hasTriedMasterFlag;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void incrementNumberOfPreparedExecutes() {
        if (getGatherPerformanceMetrics()) {
            this.numberOfPreparedExecutes++;
            this.numberOfQueriesIssued++;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void incrementNumberOfPrepares() {
        if (getGatherPerformanceMetrics()) {
            this.numberOfPrepares++;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void incrementNumberOfResultSetsCreated() {
        if (getGatherPerformanceMetrics()) {
            this.numberOfResultSetsCreated++;
        }
    }

    private void initializeDriverProperties(Properties info) throws SQLException {
        initializeProperties(info);
        this.usePlatformCharsetConverters = getUseJvmCharsetConverters();
        this.log = LogFactory.getLogger(getLogger(), LOGGER_INSTANCE_NAME);
        if (getProfileSql() || getUseUsageAdvisor()) {
            this.eventSink = ProfileEventSink.getInstance(this);
        }
        if (getCachePreparedStatements()) {
            createPreparedStatementCaches();
        }
        if (getNoDatetimeStringSync() && getUseTimezone()) {
            throw SQLError.createSQLException("Can't enable noDatetimeSync and useTimezone configuration properties at the same time", SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE);
        }
        if (getCacheCallableStatements()) {
            this.parsedCallableStatementCache = new LRUCache(getCallableStatementCacheSize());
        }
        if (getAllowMultiQueries()) {
            setCacheResultSetMetadata(false);
        }
        if (getCacheResultSetMetadata()) {
            this.resultSetMetadataCache = new LRUCache(getMetadataCacheSize());
        }
    }

    private void initializePropsFromServer() throws SQLException {
        String defaultMetadataCharset;
        int sqlMode;
        setSessionVariables();
        if (!versionMeetsMinimum(4, 1, 0)) {
            setTransformedBitIsBoolean(false);
        }
        this.parserKnowsUnicode = versionMeetsMinimum(4, 1, 0);
        if (getUseServerPreparedStmts() && versionMeetsMinimum(4, 1, 0)) {
            this.useServerPreparedStmts = true;
            if (versionMeetsMinimum(5, 0, 0) && !versionMeetsMinimum(5, 0, 3)) {
                this.useServerPreparedStmts = false;
            }
        }
        this.serverVariables.clear();
        if (versionMeetsMinimum(3, 21, 22)) {
            loadServerVariables();
            buildCollationMapping();
            LicenseConfiguration.checkLicenseType(this.serverVariables);
            String lowerCaseTables = (String) this.serverVariables.get("lower_case_table_names");
            this.lowerCaseTableNames = "on".equalsIgnoreCase(lowerCaseTables) || "1".equalsIgnoreCase(lowerCaseTables) || "2".equalsIgnoreCase(lowerCaseTables);
            configureTimezone();
            if (this.serverVariables.containsKey("max_allowed_packet")) {
                this.maxAllowedPacket = Integer.parseInt((String) this.serverVariables.get("max_allowed_packet"));
                int preferredBlobSendChunkSize = getBlobSendChunkSize();
                int allowedBlobSendChunkSize = (Math.min(preferredBlobSendChunkSize, this.maxAllowedPacket) - 8192) - 11;
                setBlobSendChunkSize(String.valueOf(allowedBlobSendChunkSize));
            }
            if (this.serverVariables.containsKey("net_buffer_length")) {
                this.netBufferLength = Integer.parseInt((String) this.serverVariables.get("net_buffer_length"));
            }
            checkTransactionIsolationLevel();
            if (!versionMeetsMinimum(4, 1, 0)) {
                checkServerEncoding();
            }
            this.io.checkForCharsetMismatch();
            if (this.serverVariables.containsKey("sql_mode")) {
                String sqlModeAsString = (String) this.serverVariables.get("sql_mode");
                try {
                    sqlMode = Integer.parseInt(sqlModeAsString);
                } catch (NumberFormatException e) {
                    sqlMode = 0;
                    if (sqlModeAsString != null) {
                        if (sqlModeAsString.indexOf("ANSI_QUOTES") != -1) {
                            sqlMode = 0 | 4;
                        }
                        if (sqlModeAsString.indexOf("NO_BACKSLASH_ESCAPES") != -1) {
                            this.noBackslashEscapes = true;
                        }
                    }
                }
                if ((sqlMode & 4) > 0) {
                    this.useAnsiQuotes = true;
                } else {
                    this.useAnsiQuotes = false;
                }
            }
        }
        this.errorMessageEncoding = CharsetMapping.getCharacterEncodingForErrorMessages(this);
        boolean overrideDefaultAutocommit = isAutoCommitNonDefaultOnServer();
        configureClientCharacterSet();
        if (versionMeetsMinimum(3, 23, 15)) {
            this.transactionsSupported = true;
            if (!overrideDefaultAutocommit) {
                setAutoCommit(true);
            }
        } else {
            this.transactionsSupported = false;
        }
        if (versionMeetsMinimum(3, 23, 36)) {
            this.hasIsolationLevels = true;
        } else {
            this.hasIsolationLevels = false;
        }
        this.hasQuotedIdentifiers = versionMeetsMinimum(3, 23, 6);
        this.io.resetMaxBuf();
        if (this.io.versionMeetsMinimum(4, 1, 0)) {
            String characterSetResultsOnServerMysql = (String) this.serverVariables.get("character_set_results");
            if (characterSetResultsOnServerMysql == null || StringUtils.startsWithIgnoreCaseAndWs(characterSetResultsOnServerMysql, "NULL")) {
                String defaultMetadataCharsetMysql = (String) this.serverVariables.get("character_set_system");
                if (defaultMetadataCharsetMysql != null) {
                    defaultMetadataCharset = CharsetMapping.getJavaEncodingForMysqlEncoding(defaultMetadataCharsetMysql, this);
                } else {
                    defaultMetadataCharset = "UTF-8";
                }
                this.characterSetMetadata = defaultMetadataCharset;
            } else {
                this.characterSetResultsOnServer = CharsetMapping.getJavaEncodingForMysqlEncoding(characterSetResultsOnServerMysql, this);
                this.characterSetMetadata = this.characterSetResultsOnServer;
            }
        }
        if (versionMeetsMinimum(4, 1, 0) && !versionMeetsMinimum(4, 1, 10) && getAllowMultiQueries() && "ON".equalsIgnoreCase((String) this.serverVariables.get("query_cache_type")) && !"0".equalsIgnoreCase((String) this.serverVariables.get("query_cache_size"))) {
            setAllowMultiQueries(false);
        }
        setupServerForTruncationChecks();
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:21:0x0090
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    private boolean isAutoCommitNonDefaultOnServer() throws java.sql.SQLException {
        /*
            r5 = this;
            r0 = 0
            r6 = r0
            r0 = r5
            java.util.Map r0 = r0.serverVariables
            java.lang.String r1 = "init_connect"
            java.lang.Object r0 = r0.get(r1)
            java.lang.String r0 = (java.lang.String) r0
            r7 = r0
            r0 = r5
            r1 = 4
            r2 = 1
            r3 = 2
            boolean r0 = r0.versionMeetsMinimum(r1, r2, r3)
            if (r0 == 0) goto La9
            r0 = r7
            if (r0 == 0) goto La9
            r0 = r7
            int r0 = r0.length()
            if (r0 <= 0) goto La9
            r0 = r5
            boolean r0 = r0.getElideSetAutoCommits()
            if (r0 != 0) goto L97
            r0 = 0
            r8 = r0
            r0 = 0
            r9 = r0
            r0 = r5
            java.sql.Statement r0 = r0.getMetadataSafeStatement()     // Catch: java.lang.Throwable -> L68
            r9 = r0
            r0 = r9
            java.lang.String r1 = "SELECT @@session.autocommit"
            java.sql.ResultSet r0 = r0.executeQuery(r1)     // Catch: java.lang.Throwable -> L68
            r8 = r0
            r0 = r8
            boolean r0 = r0.next()     // Catch: java.lang.Throwable -> L68
            if (r0 == 0) goto L62
            r0 = r5
            r1 = r8
            r2 = 1
            boolean r1 = r1.getBoolean(r2)     // Catch: java.lang.Throwable -> L68
            r0.autoCommit = r1     // Catch: java.lang.Throwable -> L68
            r0 = r5
            boolean r0 = r0.autoCommit     // Catch: java.lang.Throwable -> L68
            r1 = 1
            if (r0 == r1) goto L62
            r0 = 1
            r6 = r0
        L62:
            r0 = jsr -> L70
        L65:
            goto L94
        L68:
            r10 = move-exception
            r0 = jsr -> L70
        L6d:
            r1 = r10
            throw r1
        L70:
            r11 = r0
            r0 = r8
            if (r0 == 0) goto L81
            r0 = r8
            r0.close()     // Catch: java.sql.SQLException -> L7f
            goto L81
        L7f:
            r12 = move-exception
        L81:
            r0 = r9
            if (r0 == 0) goto L92
            r0 = r9
            r0.close()     // Catch: java.sql.SQLException -> L90
            goto L92
        L90:
            r12 = move-exception
        L92:
            ret r11
        L94:
            goto La9
        L97:
            r0 = r5
            com.mysql.jdbc.MysqlIO r0 = r0.getIO()
            r1 = 1
            boolean r0 = r0.isSetNeededForAutoCommitMode(r1)
            if (r0 == 0) goto La9
            r0 = r5
            r1 = 0
            r0.autoCommit = r1
            r0 = 1
            r6 = r0
        La9:
            r0 = r6
            return r0
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.isAutoCommitNonDefaultOnServer():boolean");
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean isClientTzUTC() {
        return this.isClientTzUTC;
    }

    @Override // java.sql.Connection
    public boolean isClosed() {
        return this.isClosed;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean isCursorFetchEnabled() throws SQLException {
        return versionMeetsMinimum(5, 0, 2) && getUseCursorFetch();
    }

    @Override // com.mysql.jdbc.Connection
    public boolean isInGlobalTx() {
        return this.isInGlobalTx;
    }

    @Override // com.mysql.jdbc.Connection
    public synchronized boolean isMasterConnection() {
        return !this.failedOver;
    }

    @Override // com.mysql.jdbc.Connection
    public boolean isNoBackslashEscapesSet() {
        return this.noBackslashEscapes;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean isReadInfoMsgEnabled() {
        return this.readInfoMsg;
    }

    @Override // java.sql.Connection
    public boolean isReadOnly() throws SQLException {
        return this.readOnly;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean isRunningOnJDK13() {
        return this.isRunningOnJDK13;
    }

    public synchronized boolean isSameResource(Connection otherConnection) {
        if (otherConnection == null) {
            return false;
        }
        boolean directCompare = true;
        String otherHost = ((ConnectionImpl) otherConnection).origHostToConnectTo;
        String otherOrigDatabase = ((ConnectionImpl) otherConnection).origDatabaseToConnectTo;
        String otherCurrentCatalog = ((ConnectionImpl) otherConnection).database;
        if (!nullSafeCompare(otherHost, this.origHostToConnectTo)) {
            directCompare = false;
        } else if (otherHost != null && otherHost.indexOf(44) == -1 && otherHost.indexOf(58) == -1) {
            directCompare = ((ConnectionImpl) otherConnection).origPortToConnectTo == this.origPortToConnectTo;
        }
        if (directCompare) {
            if (!nullSafeCompare(otherOrigDatabase, this.origDatabaseToConnectTo)) {
                directCompare = false;
            } else if (!nullSafeCompare(otherCurrentCatalog, this.database)) {
                directCompare = false;
            }
        }
        if (directCompare) {
            return true;
        }
        String otherResourceId = ((ConnectionImpl) otherConnection).getResourceId();
        String myResourceId = getResourceId();
        if ((otherResourceId != null || myResourceId != null) && nullSafeCompare(otherResourceId, myResourceId)) {
            return true;
        }
        return false;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean isServerTzUTC() {
        return this.isServerTzUTC;
    }

    /*  JADX ERROR: JadxRuntimeException in pass: BlockProcessor
        jadx.core.utils.exceptions.JadxRuntimeException: Unreachable block: B:42:0x00ca
        	at jadx.core.dex.visitors.blocks.BlockProcessor.checkForUnreachableBlocks(BlockProcessor.java:88)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.processBlocksTree(BlockProcessor.java:52)
        	at jadx.core.dex.visitors.blocks.BlockProcessor.visit(BlockProcessor.java:44)
        */
    private void loadServerVariables() throws java.sql.SQLException {
        /*
            Method dump skipped, instructions count: 207
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: com.mysql.jdbc.ConnectionImpl.loadServerVariables():void");
    }

    @Override // com.mysql.jdbc.Connection
    public boolean lowerCaseTableNames() {
        return this.lowerCaseTableNames;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void maxRowsChanged(StatementImpl stmt) {
        synchronized (this.mutex) {
            if (this.statementsUsingMaxRows == null) {
                this.statementsUsingMaxRows = new HashMap();
            }
            this.statementsUsingMaxRows.put(stmt, stmt);
            this.maxRowsChanged = true;
        }
    }

    @Override // java.sql.Connection
    public String nativeSQL(String sql) throws SQLException {
        if (sql == null) {
            return null;
        }
        Object escapedSqlResult = EscapeProcessor.escapeSQL(sql, serverSupportsConvertFn(), this);
        if (escapedSqlResult instanceof String) {
            return (String) escapedSqlResult;
        }
        return ((EscapeProcessorResult) escapedSqlResult).escapedSql;
    }

    private CallableStatement parseCallableStatement(String sql) throws SQLException {
        String parsedSql;
        boolean isFunctionCall;
        Object escapedSqlResult = EscapeProcessor.escapeSQL(sql, serverSupportsConvertFn(), this);
        if (escapedSqlResult instanceof EscapeProcessorResult) {
            parsedSql = ((EscapeProcessorResult) escapedSqlResult).escapedSql;
            isFunctionCall = ((EscapeProcessorResult) escapedSqlResult).callingStoredFunction;
        } else {
            parsedSql = (String) escapedSqlResult;
            isFunctionCall = false;
        }
        return CallableStatement.getInstance(this, parsedSql, this.database, isFunctionCall);
    }

    @Override // com.mysql.jdbc.Connection
    public boolean parserKnowsUnicode() {
        return this.parserKnowsUnicode;
    }

    @Override // com.mysql.jdbc.Connection
    public void ping() throws SQLException {
        pingInternal(true);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void pingInternal(boolean checkForClosedConnection) throws SQLException {
        if (checkForClosedConnection) {
            checkClosed();
        }
        this.io.sendCommand(14, null, null, false, null);
    }

    @Override // java.sql.Connection
    public java.sql.CallableStatement prepareCall(String sql) throws SQLException {
        return prepareCall(sql, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS);
    }

    @Override // java.sql.Connection
    public java.sql.CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        CallableStatement cStmt;
        if (versionMeetsMinimum(5, 0, 0)) {
            if (!getCacheCallableStatements()) {
                cStmt = parseCallableStatement(sql);
            } else {
                synchronized (this.parsedCallableStatementCache) {
                    CompoundCacheKey key = new CompoundCacheKey(this, getCatalog(), sql);
                    CallableStatement.CallableStatementParamInfo cachedParamInfo = (CallableStatement.CallableStatementParamInfo) this.parsedCallableStatementCache.get(key);
                    if (cachedParamInfo != null) {
                        cStmt = CallableStatement.getInstance(this, cachedParamInfo);
                    } else {
                        cStmt = parseCallableStatement(sql);
                        this.parsedCallableStatementCache.put(key, cStmt.paramInfo);
                    }
                }
            }
            cStmt.setResultSetType(resultSetType);
            cStmt.setResultSetConcurrency(resultSetConcurrency);
            return cStmt;
        }
        throw SQLError.createSQLException("Callable statements not supported.", SQLError.SQL_STATE_DRIVER_NOT_CAPABLE);
    }

    @Override // java.sql.Connection
    public java.sql.CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        if (getPedantic() && resultSetHoldability != 1) {
            throw SQLError.createSQLException("HOLD_CUSRORS_OVER_COMMIT is only supported holdability level", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        CallableStatement cStmt = (CallableStatement) prepareCall(sql, resultSetType, resultSetConcurrency);
        return cStmt;
    }

    @Override // java.sql.Connection
    public java.sql.PreparedStatement prepareStatement(String sql) throws SQLException {
        return prepareStatement(sql, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS);
    }

    @Override // java.sql.Connection
    public java.sql.PreparedStatement prepareStatement(String sql, int autoGenKeyIndex) throws SQLException {
        java.sql.PreparedStatement pStmt = prepareStatement(sql);
        ((PreparedStatement) pStmt).setRetrieveGeneratedKeys(autoGenKeyIndex == 1);
        return pStmt;
    }

    @Override // java.sql.Connection
    public java.sql.PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        PreparedStatement pStmt;
        checkClosed();
        boolean canServerPrepare = true;
        String nativeSql = getProcessEscapeCodesForPrepStmts() ? nativeSQL(sql) : sql;
        if (getEmulateUnsupportedPstmts()) {
            canServerPrepare = canHandleAsServerPreparedStatement(nativeSql);
        }
        if (this.useServerPreparedStmts && canServerPrepare) {
            if (getCachePreparedStatements()) {
                synchronized (this.serverSideStatementCache) {
                    pStmt = (ServerPreparedStatement) this.serverSideStatementCache.remove(sql);
                    if (pStmt != null) {
                        ((ServerPreparedStatement) pStmt).setClosed(false);
                        pStmt.clearParameters();
                    }
                    if (pStmt == null) {
                        try {
                            pStmt = ServerPreparedStatement.getInstance(this, nativeSql, this.database, resultSetType, resultSetConcurrency);
                            if (sql.length() < getPreparedStatementCacheSqlLimit()) {
                                ((ServerPreparedStatement) pStmt).isCached = true;
                            }
                            pStmt.setResultSetType(resultSetType);
                            pStmt.setResultSetConcurrency(resultSetConcurrency);
                        } catch (SQLException sqlEx) {
                            if (getEmulateUnsupportedPstmts()) {
                                pStmt = clientPrepareStatement(nativeSql, resultSetType, resultSetConcurrency, false);
                                if (sql.length() < getPreparedStatementCacheSqlLimit()) {
                                    this.serverSideStatementCheckCache.put(sql, Boolean.FALSE);
                                }
                            } else {
                                throw sqlEx;
                            }
                        }
                    }
                }
            } else {
                try {
                    pStmt = ServerPreparedStatement.getInstance(this, nativeSql, this.database, resultSetType, resultSetConcurrency);
                    pStmt.setResultSetType(resultSetType);
                    pStmt.setResultSetConcurrency(resultSetConcurrency);
                } catch (SQLException sqlEx2) {
                    if (getEmulateUnsupportedPstmts()) {
                        pStmt = clientPrepareStatement(nativeSql, resultSetType, resultSetConcurrency, false);
                    } else {
                        throw sqlEx2;
                    }
                }
            }
        } else {
            pStmt = clientPrepareStatement(nativeSql, resultSetType, resultSetConcurrency, false);
        }
        return pStmt;
    }

    @Override // java.sql.Connection
    public java.sql.PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        if (getPedantic() && resultSetHoldability != 1) {
            throw SQLError.createSQLException("HOLD_CUSRORS_OVER_COMMIT is only supported holdability level", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        return prepareStatement(sql, resultSetType, resultSetConcurrency);
    }

    @Override // java.sql.Connection
    public java.sql.PreparedStatement prepareStatement(String sql, int[] autoGenKeyIndexes) throws SQLException {
        java.sql.PreparedStatement pStmt = prepareStatement(sql);
        ((PreparedStatement) pStmt).setRetrieveGeneratedKeys(autoGenKeyIndexes != null && autoGenKeyIndexes.length > 0);
        return pStmt;
    }

    @Override // java.sql.Connection
    public java.sql.PreparedStatement prepareStatement(String sql, String[] autoGenKeyColNames) throws SQLException {
        java.sql.PreparedStatement pStmt = prepareStatement(sql);
        ((PreparedStatement) pStmt).setRetrieveGeneratedKeys(autoGenKeyColNames != null && autoGenKeyColNames.length > 0);
        return pStmt;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void realClose(boolean calledExplicitly, boolean issueRollback, boolean skipLocalTeardown, Throwable reason) throws SQLException {
        SQLException sqlEx = null;
        if (isClosed()) {
            return;
        }
        this.forceClosedReason = reason;
        try {
            if (!skipLocalTeardown) {
                if (!getAutoCommit() && issueRollback) {
                    try {
                        rollback();
                    } catch (SQLException ex) {
                        sqlEx = ex;
                    }
                }
                reportMetrics();
                if (getUseUsageAdvisor()) {
                    if (!calledExplicitly) {
                        this.eventSink.consumeEvent(new ProfilerEvent((byte) 0, "", getCatalog(), getId(), -1, -1, System.currentTimeMillis(), 0L, Constants.MILLIS_I18N, (String) null, this.pointOfOrigin, "Connection implicitly closed by Driver. You should call Connection.close() from your code to free resources more efficiently and avoid resource leaks."));
                    }
                    long connectionLifeTime = System.currentTimeMillis() - this.connectionCreationTimeMillis;
                    if (connectionLifeTime < 500) {
                        this.eventSink.consumeEvent(new ProfilerEvent((byte) 0, "", getCatalog(), getId(), -1, -1, System.currentTimeMillis(), 0L, Constants.MILLIS_I18N, (String) null, this.pointOfOrigin, "Connection lifetime of < .5 seconds. You might be un-necessarily creating short-lived connections and should investigate connection pooling to be more efficient."));
                    }
                }
                try {
                    closeAllOpenStatements();
                } catch (SQLException ex2) {
                    sqlEx = ex2;
                }
                if (this.io != null) {
                    try {
                        this.io.quit();
                    } catch (Exception e) {
                    }
                }
            } else {
                this.io.forceClose();
            }
            if (sqlEx != null) {
                throw sqlEx;
            }
        } finally {
            this.openStatements = null;
            this.io = null;
            ProfileEventSink.removeInstance(this);
            this.isClosed = true;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void recachePreparedStatement(ServerPreparedStatement pstmt) throws SQLException {
        if (pstmt.isPoolable()) {
            synchronized (this.serverSideStatementCache) {
                this.serverSideStatementCache.put(pstmt.originalSql, pstmt);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void registerQueryExecutionTime(long queryTimeMs) {
        if (queryTimeMs > this.longestQueryTimeMs) {
            this.longestQueryTimeMs = queryTimeMs;
            repartitionPerformanceHistogram();
        }
        addToPerformanceHistogram(queryTimeMs, 1);
        if (queryTimeMs < this.shortestQueryTimeMs) {
            this.shortestQueryTimeMs = queryTimeMs == 0 ? 1L : queryTimeMs;
        }
        this.numberOfQueriesIssued++;
        this.totalQueryTimeMs += queryTimeMs;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void registerStatement(StatementImpl stmt) {
        synchronized (this.openStatements) {
            this.openStatements.put(stmt, stmt);
        }
    }

    @Override // java.sql.Connection
    public void releaseSavepoint(Savepoint arg0) throws SQLException {
    }

    private void repartitionHistogram(int[] histCounts, long[] histBreakpoints, long currentLowerBound, long currentUpperBound) {
        if (this.oldHistCounts == null) {
            this.oldHistCounts = new int[histCounts.length];
            this.oldHistBreakpoints = new long[histBreakpoints.length];
        }
        System.arraycopy(histCounts, 0, this.oldHistCounts, 0, histCounts.length);
        System.arraycopy(histBreakpoints, 0, this.oldHistBreakpoints, 0, histBreakpoints.length);
        createInitialHistogram(histBreakpoints, currentLowerBound, currentUpperBound);
        for (int i = 0; i < HISTOGRAM_BUCKETS; i++) {
            addToHistogram(histCounts, histBreakpoints, this.oldHistBreakpoints[i], this.oldHistCounts[i], currentLowerBound, currentUpperBound);
        }
    }

    private void repartitionPerformanceHistogram() {
        checkAndCreatePerformanceHistogram();
        repartitionHistogram(this.perfMetricsHistCounts, this.perfMetricsHistBreakpoints, this.shortestQueryTimeMs == Long.MAX_VALUE ? 0L : this.shortestQueryTimeMs, this.longestQueryTimeMs);
    }

    private void repartitionTablesAccessedHistogram() {
        checkAndCreateTablesAccessedHistogram();
        repartitionHistogram(this.numTablesMetricsHistCounts, this.numTablesMetricsHistBreakpoints, this.minimumNumberTablesAccessed == Long.MAX_VALUE ? 0L : this.minimumNumberTablesAccessed, this.maximumNumberTablesAccessed);
    }

    private void reportMetrics() {
        if (getGatherPerformanceMetrics()) {
            StringBuffer logMessage = new StringBuffer(256);
            logMessage.append("** Performance Metrics Report **\n");
            logMessage.append(new StringBuffer().append("\nLongest reported query: ").append(this.longestQueryTimeMs).append(" ms").toString());
            logMessage.append(new StringBuffer().append("\nShortest reported query: ").append(this.shortestQueryTimeMs).append(" ms").toString());
            logMessage.append(new StringBuffer().append("\nAverage query execution time: ").append(this.totalQueryTimeMs / this.numberOfQueriesIssued).append(" ms").toString());
            logMessage.append(new StringBuffer().append("\nNumber of statements executed: ").append(this.numberOfQueriesIssued).toString());
            logMessage.append(new StringBuffer().append("\nNumber of result sets created: ").append(this.numberOfResultSetsCreated).toString());
            logMessage.append(new StringBuffer().append("\nNumber of statements prepared: ").append(this.numberOfPrepares).toString());
            logMessage.append(new StringBuffer().append("\nNumber of prepared statement executions: ").append(this.numberOfPreparedExecutes).toString());
            if (this.perfMetricsHistBreakpoints != null) {
                logMessage.append("\n\n\tTiming Histogram:\n");
                int highestCount = Integer.MIN_VALUE;
                for (int i = 0; i < HISTOGRAM_BUCKETS; i++) {
                    if (this.perfMetricsHistCounts[i] > highestCount) {
                        highestCount = this.perfMetricsHistCounts[i];
                    }
                }
                if (highestCount == 0) {
                    highestCount = 1;
                }
                for (int i2 = 0; i2 < 19; i2++) {
                    if (i2 == 0) {
                        logMessage.append(new StringBuffer().append("\n\tless than ").append(this.perfMetricsHistBreakpoints[i2 + 1]).append(" ms: \t").append(this.perfMetricsHistCounts[i2]).toString());
                    } else {
                        logMessage.append(new StringBuffer().append("\n\tbetween ").append(this.perfMetricsHistBreakpoints[i2]).append(" and ").append(this.perfMetricsHistBreakpoints[i2 + 1]).append(" ms: \t").append(this.perfMetricsHistCounts[i2]).toString());
                    }
                    logMessage.append("\t");
                    int numPointsToGraph = (int) (HISTOGRAM_BUCKETS * (this.perfMetricsHistCounts[i2] / highestCount));
                    for (int j = 0; j < numPointsToGraph; j++) {
                        logMessage.append("*");
                    }
                    if (this.longestQueryTimeMs < this.perfMetricsHistCounts[i2 + 1]) {
                        break;
                    }
                }
                if (this.perfMetricsHistBreakpoints[18] < this.longestQueryTimeMs) {
                    logMessage.append("\n\tbetween ");
                    logMessage.append(this.perfMetricsHistBreakpoints[18]);
                    logMessage.append(" and ");
                    logMessage.append(this.perfMetricsHistBreakpoints[19]);
                    logMessage.append(" ms: \t");
                    logMessage.append(this.perfMetricsHistCounts[19]);
                }
            }
            if (this.numTablesMetricsHistBreakpoints != null) {
                logMessage.append("\n\n\tTable Join Histogram:\n");
                int highestCount2 = Integer.MIN_VALUE;
                for (int i3 = 0; i3 < HISTOGRAM_BUCKETS; i3++) {
                    if (this.numTablesMetricsHistCounts[i3] > highestCount2) {
                        highestCount2 = this.numTablesMetricsHistCounts[i3];
                    }
                }
                if (highestCount2 == 0) {
                    highestCount2 = 1;
                }
                for (int i4 = 0; i4 < 19; i4++) {
                    if (i4 == 0) {
                        logMessage.append(new StringBuffer().append("\n\t").append(this.numTablesMetricsHistBreakpoints[i4 + 1]).append(" tables or less: \t\t").append(this.numTablesMetricsHistCounts[i4]).toString());
                    } else {
                        logMessage.append(new StringBuffer().append("\n\tbetween ").append(this.numTablesMetricsHistBreakpoints[i4]).append(" and ").append(this.numTablesMetricsHistBreakpoints[i4 + 1]).append(" tables: \t").append(this.numTablesMetricsHistCounts[i4]).toString());
                    }
                    logMessage.append("\t");
                    int numPointsToGraph2 = (int) (HISTOGRAM_BUCKETS * (this.numTablesMetricsHistCounts[i4] / highestCount2));
                    for (int j2 = 0; j2 < numPointsToGraph2; j2++) {
                        logMessage.append("*");
                    }
                    if (this.maximumNumberTablesAccessed < this.numTablesMetricsHistBreakpoints[i4 + 1]) {
                        break;
                    }
                }
                if (this.numTablesMetricsHistBreakpoints[18] < this.maximumNumberTablesAccessed) {
                    logMessage.append("\n\tbetween ");
                    logMessage.append(this.numTablesMetricsHistBreakpoints[18]);
                    logMessage.append(" and ");
                    logMessage.append(this.numTablesMetricsHistBreakpoints[19]);
                    logMessage.append(" tables: ");
                    logMessage.append(this.numTablesMetricsHistCounts[19]);
                }
            }
            this.log.logInfo(logMessage);
            this.metricsLastReportedMs = System.currentTimeMillis();
        }
    }

    private void reportMetricsIfNeeded() {
        if (getGatherPerformanceMetrics() && System.currentTimeMillis() - this.metricsLastReportedMs > getReportMetricsIntervalMillis()) {
            reportMetrics();
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void reportNumberOfTablesAccessed(int numTablesAccessed) {
        if (numTablesAccessed < this.minimumNumberTablesAccessed) {
            this.minimumNumberTablesAccessed = numTablesAccessed;
        }
        if (numTablesAccessed > this.maximumNumberTablesAccessed) {
            this.maximumNumberTablesAccessed = numTablesAccessed;
            repartitionTablesAccessedHistogram();
        }
        addToTablesAccessedHistogram(numTablesAccessed, 1);
    }

    @Override // com.mysql.jdbc.Connection
    public void resetServerState() throws SQLException {
        if (!getParanoid() && this.io != null && versionMeetsMinimum(4, 0, 6)) {
            changeUser(this.user, this.password);
        }
    }

    /* JADX WARN: Finally extract failed */
    @Override // java.sql.Connection
    public void rollback() throws SQLException {
        synchronized (getMutex()) {
            try {
                checkClosed();
                try {
                    if (this.autoCommit && !getRelaxAutoCommit()) {
                        throw SQLError.createSQLException("Can't call rollback when autocommit=true", SQLError.SQL_STATE_CONNECTION_NOT_OPEN);
                    }
                    if (this.transactionsSupported) {
                        try {
                            rollbackNoChecks();
                        } catch (SQLException sqlEx) {
                            if (getIgnoreNonTxTables() && sqlEx.getErrorCode() != 1196) {
                                throw sqlEx;
                            }
                        }
                    }
                    this.needsPing = getReconnectAtTxEnd();
                } catch (SQLException sqlException) {
                    if (SQLError.SQL_STATE_COMMUNICATION_LINK_FAILURE.equals(sqlException.getSQLState())) {
                        throw SQLError.createSQLException("Communications link failure during rollback(). Transaction resolution unknown.", "08007");
                    }
                    throw sqlException;
                }
            } catch (Throwable th) {
                this.needsPing = getReconnectAtTxEnd();
                throw th;
            }
        }
    }

    /* JADX WARN: Finally extract failed */
    @Override // java.sql.Connection
    public void rollback(Savepoint savepoint) throws SQLException {
        String msg;
        if (versionMeetsMinimum(4, 0, 14) || versionMeetsMinimum(4, 1, 1)) {
            synchronized (getMutex()) {
                checkClosed();
                try {
                    StringBuffer rollbackQuery = new StringBuffer("ROLLBACK TO SAVEPOINT ");
                    rollbackQuery.append('`');
                    rollbackQuery.append(savepoint.getSavepointName());
                    rollbackQuery.append('`');
                    try {
                        try {
                            java.sql.Statement stmt = createStatement();
                            stmt.executeUpdate(rollbackQuery.toString());
                            closeStatement(stmt);
                            this.needsPing = getReconnectAtTxEnd();
                        } catch (Throwable th) {
                            closeStatement(null);
                            throw th;
                        }
                    } catch (SQLException sqlEx) {
                        int errno = sqlEx.getErrorCode();
                        if (errno == 1181 && (msg = sqlEx.getMessage()) != null) {
                            int indexOfError153 = msg.indexOf("153");
                            if (indexOfError153 != -1) {
                                throw SQLError.createSQLException(new StringBuffer().append("Savepoint '").append(savepoint.getSavepointName()).append("' does not exist").toString(), SQLError.SQL_STATE_ILLEGAL_ARGUMENT, errno);
                            }
                        }
                        if (getIgnoreNonTxTables() && sqlEx.getErrorCode() != 1196) {
                            throw sqlEx;
                        }
                        if (SQLError.SQL_STATE_COMMUNICATION_LINK_FAILURE.equals(sqlEx.getSQLState())) {
                            throw SQLError.createSQLException("Communications link failure during rollback(). Transaction resolution unknown.", "08007");
                        }
                        throw sqlEx;
                    }
                } catch (Throwable th2) {
                    this.needsPing = getReconnectAtTxEnd();
                    throw th2;
                }
            }
            return;
        }
        throw new NotImplemented();
    }

    private void rollbackNoChecks() throws SQLException {
        if (getUseLocalSessionState() && versionMeetsMinimum(5, 0, 0) && !this.io.inTransactionOnServer()) {
            return;
        }
        execSQL(null, "rollback", -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
    }

    @Override // com.mysql.jdbc.Connection
    public ServerPreparedStatement serverPrepareStatement(String sql) throws SQLException {
        String nativeSql = getProcessEscapeCodesForPrepStmts() ? nativeSQL(sql) : sql;
        return ServerPreparedStatement.getInstance(this, nativeSql, getCatalog(), MysqlErrorNumbers.ER_CANT_CREATE_TABLE, MysqlErrorNumbers.ER_DB_CREATE_EXISTS);
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement serverPrepareStatement(String sql, int autoGenKeyIndex) throws SQLException {
        String nativeSql = getProcessEscapeCodesForPrepStmts() ? nativeSQL(sql) : sql;
        PreparedStatement pStmt = ServerPreparedStatement.getInstance(this, nativeSql, getCatalog(), MysqlErrorNumbers.ER_CANT_CREATE_TABLE, MysqlErrorNumbers.ER_DB_CREATE_EXISTS);
        pStmt.setRetrieveGeneratedKeys(autoGenKeyIndex == 1);
        return pStmt;
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement serverPrepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        String nativeSql = getProcessEscapeCodesForPrepStmts() ? nativeSQL(sql) : sql;
        return ServerPreparedStatement.getInstance(this, nativeSql, getCatalog(), resultSetType, resultSetConcurrency);
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement serverPrepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        if (getPedantic() && resultSetHoldability != 1) {
            throw SQLError.createSQLException("HOLD_CUSRORS_OVER_COMMIT is only supported holdability level", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
        }
        return serverPrepareStatement(sql, resultSetType, resultSetConcurrency);
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement serverPrepareStatement(String sql, int[] autoGenKeyIndexes) throws SQLException {
        PreparedStatement pStmt = serverPrepareStatement(sql);
        pStmt.setRetrieveGeneratedKeys(autoGenKeyIndexes != null && autoGenKeyIndexes.length > 0);
        return pStmt;
    }

    @Override // com.mysql.jdbc.Connection
    public java.sql.PreparedStatement serverPrepareStatement(String sql, String[] autoGenKeyColNames) throws SQLException {
        PreparedStatement pStmt = serverPrepareStatement(sql);
        pStmt.setRetrieveGeneratedKeys(autoGenKeyColNames != null && autoGenKeyColNames.length > 0);
        return pStmt;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public boolean serverSupportsConvertFn() throws SQLException {
        return versionMeetsMinimum(4, 0, 2);
    }

    /* JADX WARN: Finally extract failed */
    @Override // java.sql.Connection
    public void setAutoCommit(boolean autoCommitFlag) throws SQLException {
        synchronized (getMutex()) {
            checkClosed();
            if (getAutoReconnectForPools()) {
                setHighAvailability(true);
            }
            try {
                if (this.transactionsSupported) {
                    boolean needsSetOnServer = true;
                    if (getUseLocalSessionState() && this.autoCommit == autoCommitFlag) {
                        needsSetOnServer = false;
                    } else if (!getHighAvailability()) {
                        needsSetOnServer = getIO().isSetNeededForAutoCommitMode(autoCommitFlag);
                    }
                    this.autoCommit = autoCommitFlag;
                    if (needsSetOnServer) {
                        execSQL(null, autoCommitFlag ? "SET autocommit=1" : "SET autocommit=0", -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                    }
                } else {
                    if (!autoCommitFlag && !getRelaxAutoCommit()) {
                        throw SQLError.createSQLException("MySQL Versions Older than 3.23.15 do not support transactions", SQLError.SQL_STATE_CONNECTION_NOT_OPEN);
                    }
                    this.autoCommit = autoCommitFlag;
                }
                if (getAutoReconnectForPools()) {
                    setHighAvailability(false);
                }
            } catch (Throwable th) {
                if (getAutoReconnectForPools()) {
                    setHighAvailability(false);
                }
                throw th;
            }
        }
    }

    @Override // java.sql.Connection
    public void setCatalog(String catalog) throws SQLException {
        synchronized (getMutex()) {
            checkClosed();
            if (catalog == null) {
                throw SQLError.createSQLException("Catalog can not be null", SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
            }
            if (getUseLocalSessionState()) {
                if (this.lowerCaseTableNames) {
                    if (this.database.equalsIgnoreCase(catalog)) {
                        return;
                    }
                } else if (this.database.equals(catalog)) {
                    return;
                }
            }
            String quotedId = this.dbmd.getIdentifierQuoteString();
            if (quotedId == null || quotedId.equals(" ")) {
                quotedId = "";
            }
            StringBuffer query = new StringBuffer("USE ");
            query.append(quotedId);
            query.append(catalog);
            query.append(quotedId);
            execSQL(null, query.toString(), -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
            this.database = catalog;
        }
    }

    @Override // com.mysql.jdbc.Connection
    public synchronized void setFailedOver(boolean flag) {
        this.failedOver = flag;
    }

    private void setFailedOverState() throws SQLException {
        if (getFailOverReadOnly()) {
            setReadOnlyInternal(true);
        }
        this.queriesIssuedFailedOver = 0L;
        this.failedOver = true;
        this.masterFailTimeMillis = System.currentTimeMillis();
    }

    @Override // java.sql.Connection
    public void setHoldability(int arg0) throws SQLException {
    }

    public void setInGlobalTx(boolean flag) {
        this.isInGlobalTx = flag;
    }

    @Override // com.mysql.jdbc.Connection
    public void setPreferSlaveDuringFailover(boolean flag) {
        this.preferSlaveDuringFailover = flag;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void setReadInfoMsgEnabled(boolean flag) {
        this.readInfoMsg = flag;
    }

    @Override // java.sql.Connection
    public void setReadOnly(boolean readOnlyFlag) throws SQLException {
        checkClosed();
        if (this.failedOver && getFailOverReadOnly() && !readOnlyFlag) {
            return;
        }
        setReadOnlyInternal(readOnlyFlag);
    }

    protected void setReadOnlyInternal(boolean readOnlyFlag) throws SQLException {
        this.readOnly = readOnlyFlag;
    }

    @Override // java.sql.Connection
    public Savepoint setSavepoint() throws SQLException {
        MysqlSavepoint savepoint = new MysqlSavepoint();
        setSavepoint(savepoint);
        return savepoint;
    }

    private void setSavepoint(MysqlSavepoint savepoint) throws SQLException {
        if (versionMeetsMinimum(4, 0, 14) || versionMeetsMinimum(4, 1, 1)) {
            synchronized (getMutex()) {
                checkClosed();
                StringBuffer savePointQuery = new StringBuffer("SAVEPOINT ");
                savePointQuery.append('`');
                savePointQuery.append(savepoint.getSavepointName());
                savePointQuery.append('`');
                java.sql.Statement stmt = null;
                try {
                    stmt = createStatement();
                    stmt.executeUpdate(savePointQuery.toString());
                    closeStatement(stmt);
                } catch (Throwable th) {
                    closeStatement(stmt);
                    throw th;
                }
            }
            return;
        }
        throw new NotImplemented();
    }

    @Override // java.sql.Connection
    public synchronized Savepoint setSavepoint(String name) throws SQLException {
        MysqlSavepoint savepoint = new MysqlSavepoint(name);
        setSavepoint(savepoint);
        return savepoint;
    }

    private void setSessionVariables() throws SQLException {
        if (versionMeetsMinimum(4, 0, 0) && getSessionVariables() != null) {
            List variablesToSet = StringUtils.split(getSessionVariables(), ",", "\"'", "\"'", false);
            int numVariablesToSet = variablesToSet.size();
            java.sql.Statement stmt = null;
            try {
                stmt = getMetadataSafeStatement();
                for (int i = 0; i < numVariablesToSet; i++) {
                    String variableValuePair = (String) variablesToSet.get(i);
                    if (variableValuePair.startsWith("@")) {
                        stmt.executeUpdate(new StringBuffer().append("SET ").append(variableValuePair).toString());
                    } else {
                        stmt.executeUpdate(new StringBuffer().append("SET SESSION ").append(variableValuePair).toString());
                    }
                }
                if (stmt != null) {
                    stmt.close();
                }
            } catch (Throwable th) {
                if (stmt != null) {
                    stmt.close();
                }
                throw th;
            }
        }
    }

    @Override // java.sql.Connection
    public synchronized void setTransactionIsolation(int level) throws SQLException {
        String sql;
        checkClosed();
        if (this.hasIsolationLevels) {
            boolean shouldSendSet = false;
            if (getAlwaysSendSetIsolation()) {
                shouldSendSet = true;
            } else if (level != this.isolationLevel) {
                shouldSendSet = true;
            }
            if (getUseLocalSessionState()) {
                shouldSendSet = this.isolationLevel != level;
            }
            if (shouldSendSet) {
                switch (level) {
                    case 0:
                        throw SQLError.createSQLException("Transaction isolation level NONE not supported by MySQL");
                    case 1:
                        sql = "SET SESSION TRANSACTION ISOLATION LEVEL READ UNCOMMITTED";
                        break;
                    case 2:
                        sql = "SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED";
                        break;
                    case 3:
                    case 5:
                    case 6:
                    case 7:
                    default:
                        throw SQLError.createSQLException(new StringBuffer().append("Unsupported transaction isolation level '").append(level).append("'").toString(), SQLError.SQL_STATE_DRIVER_NOT_CAPABLE);
                    case 4:
                        sql = "SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ";
                        break;
                    case 8:
                        sql = "SET SESSION TRANSACTION ISOLATION LEVEL SERIALIZABLE";
                        break;
                }
                execSQL(null, sql, -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                this.isolationLevel = level;
                return;
            }
            return;
        }
        throw SQLError.createSQLException("Transaction Isolation Levels are not supported on MySQL versions older than 3.23.36.", SQLError.SQL_STATE_DRIVER_NOT_CAPABLE);
    }

    @Override // java.sql.Connection
    public synchronized void setTypeMap(Map map) throws SQLException {
        this.typeMap = map;
    }

    private void setupServerForTruncationChecks() throws SQLException {
        if (getJdbcCompliantTruncation() && versionMeetsMinimum(5, 0, 2)) {
            String currentSqlMode = (String) this.serverVariables.get("sql_mode");
            boolean strictTransTablesIsSet = StringUtils.indexOfIgnoreCase(currentSqlMode, "STRICT_TRANS_TABLES") != -1;
            if (currentSqlMode == null || currentSqlMode.length() == 0 || !strictTransTablesIsSet) {
                StringBuffer commandBuf = new StringBuffer("SET sql_mode='");
                if (currentSqlMode != null && currentSqlMode.length() > 0) {
                    commandBuf.append(currentSqlMode);
                    commandBuf.append(",");
                }
                commandBuf.append("STRICT_TRANS_TABLES'");
                execSQL(null, commandBuf.toString(), -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                setJdbcCompliantTruncation(false);
                return;
            }
            if (strictTransTablesIsSet) {
                setJdbcCompliantTruncation(false);
            }
        }
    }

    private boolean shouldFallBack() {
        long secondsSinceFailedOver = (System.currentTimeMillis() - this.masterFailTimeMillis) / 1000;
        boolean tryFallback = secondsSinceFailedOver >= ((long) getSecondsBeforeRetryMaster()) || this.queriesIssuedFailedOver >= ((long) getQueriesBeforeRetryMaster());
        return tryFallback;
    }

    @Override // com.mysql.jdbc.Connection
    public void shutdownServer() throws SQLException {
        try {
            this.io.sendCommand(8, null, null, false, null);
        } catch (Exception ex) {
            throw SQLError.createSQLException(new StringBuffer().append("Unhandled exception '").append(ex.toString()).append("'").toString(), SQLError.SQL_STATE_GENERAL_ERROR);
        }
    }

    @Override // com.mysql.jdbc.Connection
    public boolean supportsIsolationLevel() {
        return this.hasIsolationLevels;
    }

    @Override // com.mysql.jdbc.Connection
    public boolean supportsQuotedIdentifiers() {
        return this.hasQuotedIdentifiers;
    }

    @Override // com.mysql.jdbc.Connection
    public boolean supportsTransactions() {
        return this.transactionsSupported;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void unregisterStatement(StatementImpl stmt) {
        if (this.openStatements != null) {
            synchronized (this.openStatements) {
                this.openStatements.remove(stmt);
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public void unsetMaxRows(StatementImpl stmt) throws SQLException {
        synchronized (this.mutex) {
            if (this.statementsUsingMaxRows != null) {
                Object found = this.statementsUsingMaxRows.remove(stmt);
                if (found != null && this.statementsUsingMaxRows.size() == 0) {
                    execSQL(null, "SET OPTION SQL_SELECT_LIMIT=DEFAULT", -1, null, MysqlErrorNumbers.ER_YES, MysqlErrorNumbers.ER_DB_CREATE_EXISTS, false, this.database, null, false);
                    this.maxRowsChanged = false;
                }
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean useAnsiQuotedIdentifiers() {
        return this.useAnsiQuotes;
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    public boolean useMaxRows() {
        boolean z;
        synchronized (this.mutex) {
            z = this.maxRowsChanged;
        }
        return z;
    }

    @Override // com.mysql.jdbc.Connection
    public boolean versionMeetsMinimum(int major, int minor, int subminor) throws SQLException {
        checkClosed();
        return this.io.versionMeetsMinimum(major, minor, subminor);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public CachedResultSetMetaData getCachedMetaData(String sql) {
        CachedResultSetMetaData cachedResultSetMetaData;
        if (this.resultSetMetadataCache != null) {
            synchronized (this.resultSetMetadataCache) {
                cachedResultSetMetaData = (CachedResultSetMetaData) this.resultSetMetadataCache.get(sql);
            }
            return cachedResultSetMetaData;
        }
        return null;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void initializeResultsMetadataFromCache(String sql, CachedResultSetMetaData cachedMetaData, ResultSetInternalMethods resultSet) throws SQLException {
        if (cachedMetaData == null) {
            CachedResultSetMetaData cachedMetaData2 = new CachedResultSetMetaData();
            resultSet.buildIndexMapping();
            resultSet.initializeWithMetadata();
            if (resultSet instanceof UpdatableResultSet) {
                ((UpdatableResultSet) resultSet).checkUpdatability();
            }
            resultSet.populateCachedMetaData(cachedMetaData2);
            this.resultSetMetadataCache.put(sql, cachedMetaData2);
            return;
        }
        resultSet.initializeFromCachedMetaData(cachedMetaData);
        resultSet.initializeWithMetadata();
        if (resultSet instanceof UpdatableResultSet) {
            ((UpdatableResultSet) resultSet).checkUpdatability();
        }
    }

    @Override // com.mysql.jdbc.Connection
    public String getStatementComment() {
        return this.statementComment;
    }

    @Override // com.mysql.jdbc.Connection
    public void setStatementComment(String comment) {
        this.statementComment = comment;
    }
}
